# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""Unit tests for the 'console' module."""

import unittest
import sys
import os
import subprocess
import sysconfig
import platform

try:
    # python2
    import StringIO as io
except ImportError:
    # python3
    import io


# handle python2 vs python3 difference
try:
    from tempfile import TemporaryDirectory  # noqa: F401
    import tempfile
except ImportError:
    # XXX: python2.7 only
    from backports import tempfile

import opentimelineio as otio
import opentimelineio.test_utils as otio_test_utils
import opentimelineio.console as otio_console

SAMPLE_DATA_DIR = os.path.join(os.path.dirname(__file__), "sample_data")
SCREENING_EXAMPLE_PATH = os.path.join(SAMPLE_DATA_DIR, "screening_example.edl")
PREMIERE_EXAMPLE_PATH = os.path.join(SAMPLE_DATA_DIR, "premiere_example.xml")
MULTITRACK_PATH = os.path.join(SAMPLE_DATA_DIR, "multitrack.otio")
TRANSITION_PATH = os.path.join(SAMPLE_DATA_DIR, "transition.otio")


def CreateShelloutTest(cl):
    if os.environ.get("OTIO_DISABLE_SHELLOUT_TESTS"):
        newSuite = None
    else:
        class newSuite(cl):
            SHELL_OUT = True
        newSuite.__name__ = cl.__name__ + "_on_shell"

    return newSuite


class ConsoleTester(otio_test_utils.OTIOAssertions):
    """ Base class for running console tests both by directly calling main() and
    by shelling out.
    """

    SHELL_OUT = False

    def setUp(self):
        self.saved_args = sys.argv
        self.old_stdout = sys.stdout
        self.old_stderr = sys.stderr
        sys.stdout = io.StringIO()
        sys.stderr = io.StringIO()

    def run_test(self):
        if self.SHELL_OUT:
            # make sure its on the path
            console_script = os.path.join(sysconfig.get_path('scripts'), sys.argv[0])
            if platform.system() == 'Windows':
                console_script += '.exe'

            if not os.path.exists(console_script):
                self.fail(
                    "Could not find '{}'.  Tests that explicitly shell"
                    " out can be disabled by setting the environment variable "
                    "OTIO_DISABLE_SHELLOUT_TESTS.".format(console_script)
                )

            # actually run the test (sys.argv is already populated correctly)
            proc = subprocess.Popen(
                sys.argv,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            )
            stdout, stderr = proc.communicate()

            # XXX 2.7 vs 3.xx bug
            if type(stdout) is not str:
                stdout = stdout.decode("utf-8")
                stderr = stderr.decode("utf-8")

            sys.stdout.write(stdout)
            sys.stderr.write(stderr)

            if proc.returncode != 0:
                raise SystemExit()
        else:
            self.test_module.main()

        # pre-fetch these strings for easy access
        stdout = sys.stdout.getvalue()
        stderr = sys.stderr.getvalue()

        if platform.system() == 'Windows':
            # Normalize line-endings for assertEqual(expected, actual)
            stdout = stdout.replace('\r\n', '\n')
            stderr = stderr.replace('\r\n', '\n')

        return stdout, stderr

    def tearDown(self):
        sys.stdout = self.old_stdout
        sys.stderr = self.old_stderr
        sys.argv = self.saved_args


class OTIOStatTest(ConsoleTester, unittest.TestCase):
    test_module = otio_console.otiostat

    def test_basic(self):
        sys.argv = ['otiostat', SCREENING_EXAMPLE_PATH]
        self.run_test()
        self.assertIn("top level object: Timeline.1", sys.stdout.getvalue())


OTIOStatTest_ShellOut = CreateShelloutTest(OTIOStatTest)


class OTIOCatTests(ConsoleTester, unittest.TestCase):
    test_module = otio_console.otiocat

    def test_basic(self):
        sys.argv = ['otiocat', SCREENING_EXAMPLE_PATH, "-a", "rate=24.0"]
        self.run_test()
        self.assertIn('"name": "Example_Screening.01",', sys.stdout.getvalue())

    def test_no_media_linker(self):
        sys.argv = ['otiocat', SCREENING_EXAMPLE_PATH, "-m", "none"]
        self.run_test()
        self.assertIn('"name": "Example_Screening.01",', sys.stdout.getvalue())

    def test_input_argument_error(self):
        sys.argv = [
            'otiocat',
            SCREENING_EXAMPLE_PATH,
            "-a", "foobar",
        ]

        with self.assertRaises(SystemExit):
            self.run_test()

        # read results back in
        self.assertIn('error: adapter', sys.stderr.getvalue())

    def test_media_linker_argument_error(self):
        sys.argv = [
            'otiocat',
            SCREENING_EXAMPLE_PATH,
            "-M", "foobar",
        ]

        with self.assertRaises(SystemExit):
            self.run_test()

        # read results back in
        self.assertIn('error: media linker', sys.stderr.getvalue())


OTIOCatTests_OnShell = CreateShelloutTest(OTIOCatTests)


class OTIOConvertTests(ConsoleTester, unittest.TestCase):
    test_module = otio_console.otioconvert

    def test_basic(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_file = os.path.join(temp_dir, "test_basic.otio")
            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                '--tracks', '0',
                "-a", "rate=24",
            ]
            self.run_test()

            # read results back in
            with open(temp_file, 'r') as fi:
                self.assertIn('"name": "Example_Screening.01",', fi.read())

    def test_begin_end(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_file = os.path.join(temp_dir, "test_begin_end.otio")

            # begin needs to be a,b
            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                "--begin", "foobar"
            ]
            with self.assertRaises(SystemExit):
                self.run_test()

            # end requires begin
            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                "--end", "foobar"
            ]
            with self.assertRaises(SystemExit):
                self.run_test()

            # prune everything
            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                "--begin", "0,24",
                "--end", "0,24",
            ]
            otio_console.otioconvert.main()

            # check that begin/end "," parsing is checked
            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                "--begin", "0",
                "--end", "0,24",
            ]
            with self.assertRaises(SystemExit):
                self.run_test()

            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                "--begin", "0,24",
                "--end", "0",
            ]
            with self.assertRaises(SystemExit):
                self.run_test()

            result = otio.adapters.read_from_file(temp_file, "otio_json")
            self.assertEqual(len(result.tracks[0]), 0)
            self.assertEqual(result.name, "Example_Screening.01")

    def test_input_argument_error(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_file = os.path.join(temp_dir, "test_input_argument_error.otio")

            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                "-a", "foobar",
            ]

            with self.assertRaises(SystemExit):
                self.run_test()

            # read results back in
            self.assertIn('error: input adapter', sys.stderr.getvalue())

    def test_output_argument_error(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_file = os.path.join(temp_dir, "test_output_argument_error.otio")

            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                "-A", "foobar",
            ]

            with self.assertRaises(SystemExit):
                self.run_test()

            # read results back in
            self.assertIn('error: output adapter', sys.stderr.getvalue())

    def test_media_linker_argument_error(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_file = os.path.join(temp_dir, "test_media_linker_argument_error.otio")

            sys.argv = [
                'otioconvert',
                '-i', SCREENING_EXAMPLE_PATH,
                '-o', temp_file,
                '-O', 'otio_json',
                "-m", "fake_linker",
                "-M", "somestring=foobar",
                "-M", "foobar",
            ]

            with self.assertRaises(SystemExit):
                self.run_test()

            # read results back in
            self.assertIn('error: media linker', sys.stderr.getvalue())


OTIOConvertTests_OnShell = CreateShelloutTest(OTIOConvertTests)


class OTIOPlugInfoTest(ConsoleTester, unittest.TestCase):
    test_module = otio_console.otiopluginfo

    def test_basic(self):
        sys.argv = ['otiopluginfo']
        self.run_test()
        self.assertIn("Manifests loaded:", sys.stdout.getvalue())


OTIOPlugInfoTest_ShellOut = CreateShelloutTest(OTIOStatTest)


class OTIOToolTest(ConsoleTester, unittest.TestCase):
    test_module = otio_console.otiotool

    def test_list_tracks(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--list-tracks'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: OTIO TEST - multitrack.Exported.01
TRACK: Sequence (Video)
TRACK: Sequence 2 (Video)
TRACK: Sequence 3 (Video)
""", out)

    def test_list_clips(self):
        sys.argv = [
            'otiotool',
            '-i', SCREENING_EXAMPLE_PATH,
            '--list-clips'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: Example_Screening.01
  CLIP: ZZ100_501 (LAY3)
  CLIP: ZZ100_502A (LAY3)
  CLIP: ZZ100_503A (LAY1)
  CLIP: ZZ100_504C (LAY1)
  CLIP: ZZ100_504B (LAY1)
  CLIP: ZZ100_507C (LAY2)
  CLIP: ZZ100_508 (LAY2)
  CLIP: ZZ100_510 (LAY1)
  CLIP: ZZ100_510B (LAY1)
""", out)

    def test_list_markers(self):
        sys.argv = [
            'otiotool',
            '-i', PREMIERE_EXAMPLE_PATH,
            '--list-markers'
        ]
        out, err = self.run_test()
        self.assertEqual(
            ("TIMELINE: sc01_sh010_layerA\n"
             "  MARKER: global: 00:00:03:23 local: 00:00:03:23 duration: 0.0 color: RED name: My MArker 1\n"  # noqa: E501 line too long
             "  MARKER: global: 00:00:16:12 local: 00:00:16:12 duration: 0.0 color: RED name: dsf\n"  # noqa: E501 line too long
             "  MARKER: global: 00:00:09:28 local: 00:00:09:28 duration: 0.0 color: RED name: \n"  # noqa: E501 line too long
             "  MARKER: global: 00:00:13:05 local: 00:00:02:13 duration: 0.0 color: RED name: \n"),  # noqa: E501 line too long
            out)

    def test_list_tracks_and_clips(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--list-tracks',
            '--list-clips'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: OTIO TEST - multitrack.Exported.01
TRACK: Sequence (Video)
  CLIP: tech.fux (loop)-HD.mp4
  CLIP: out-b (loop)-HD.mp4
  CLIP: brokchrd (loop)-HD.mp4
TRACK: Sequence 2 (Video)
  CLIP: t-hawk (loop)-HD.mp4
TRACK: Sequence 3 (Video)
  CLIP: KOLL-HD.mp4
""", out)

    def test_list_tracks_and_clips_and_media(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--list-tracks',
            '--list-clips',
            '--list-media'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: OTIO TEST - multitrack.Exported.01
TRACK: Sequence (Video)
  CLIP: tech.fux (loop)-HD.mp4
    MEDIA: None
  CLIP: out-b (loop)-HD.mp4
    MEDIA: None
  CLIP: brokchrd (loop)-HD.mp4
    MEDIA: None
TRACK: Sequence 2 (Video)
  CLIP: t-hawk (loop)-HD.mp4
    MEDIA: None
TRACK: Sequence 3 (Video)
  CLIP: KOLL-HD.mp4
    MEDIA: None
""", out)

    def test_list_tracks_and_clips_and_media_and_markers(self):
        sys.argv = [
            'otiotool',
            '-i', PREMIERE_EXAMPLE_PATH,
            '--list-tracks',
            '--list-clips',
            '--list-media',
            '--list-markers'
        ]
        out, err = self.run_test()
        self.assertEqual(
            ("TIMELINE: sc01_sh010_layerA\n"
             "  MARKER: global: 00:00:03:23 local: 00:00:03:23 duration: 0.0 color: RED name: My MArker 1\n"  # noqa E501 line too long
             "  MARKER: global: 00:00:16:12 local: 00:00:16:12 duration: 0.0 color: RED name: dsf\n"  # noqa E501 line too long
             "  MARKER: global: 00:00:09:28 local: 00:00:09:28 duration: 0.0 color: RED name: \n"  # noqa E501 line too long
             "TRACK:  (Video)\n"
             "  CLIP: sc01_sh010_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh010_anim.mov\n"
             "TRACK:  (Video)\n"
             "  CLIP: sc01_sh010_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh010_anim.mov\n"
             "  CLIP: sc01_sh020_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh020_anim.mov\n"
             "  CLIP: sc01_sh030_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh030_anim.mov\n"
             "  MARKER: global: 00:00:13:05 local: 00:00:02:13 duration: 0.0 color: RED name: \n"  # noqa E501 line too long
             "TRACK:  (Video)\n"
             "  CLIP: test_title\n"
             "    MEDIA: None\n"
             "TRACK:  (Video)\n"
             "  CLIP: sc01_master_layerA_sh030_temp.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_master_layerA_sh030_temp.mov\n"  # noqa E501 line too long
             "  CLIP: sc01_sh010_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh010_anim.mov\n"
             "TRACK:  (Audio)\n"
             "  CLIP: sc01_sh010_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh010_anim.mov\n"
             "  CLIP: sc01_sh010_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh010_anim.mov\n"
             "TRACK:  (Audio)\n"
             "  CLIP: sc01_placeholder.wav\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_placeholder.wav\n"
             "TRACK:  (Audio)\n"
             "  CLIP: track_08.wav\n"
             "    MEDIA: file://localhost/D%3a/media/track_08.wav\n"
             "TRACK:  (Audio)\n"
             "  CLIP: sc01_master_layerA_sh030_temp.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_master_layerA_sh030_temp.mov\n"  # noqa E501 line too long
             "  CLIP: sc01_sh010_anim.mov\n"
             "    MEDIA: file://localhost/D%3a/media/sc01_sh010_anim.mov\n"),
            out)

    def test_verify_media(self):
        sys.argv = [
            'otiotool',
            '-i', PREMIERE_EXAMPLE_PATH,
            '--list-tracks',
            '--list-clips',
            '--list-media',
            '--verify-media'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: sc01_sh010_layerA
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh010_anim.mov
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh010_anim.mov
  CLIP: sc01_sh020_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh020_anim.mov
  CLIP: sc01_sh030_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh030_anim.mov
TRACK:  (Video)
  CLIP: test_title
    MEDIA: None
TRACK:  (Video)
  CLIP: sc01_master_layerA_sh030_temp.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_sh010_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh010_anim.mov
  CLIP: sc01_sh010_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_placeholder.wav
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_placeholder.wav
TRACK:  (Audio)
  CLIP: track_08.wav
    MEDIA NOT FOUND: file://localhost/D%3a/media/track_08.wav
TRACK:  (Audio)
  CLIP: sc01_master_layerA_sh030_temp.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
    MEDIA NOT FOUND: file://localhost/D%3a/media/sc01_sh010_anim.mov
""", out)

    def test_video_only(self):
        sys.argv = [
            'otiotool',
            '-i', PREMIERE_EXAMPLE_PATH,
            '--video-only',
            '--list-clips'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: sc01_sh010_layerA
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh020_anim.mov
  CLIP: sc01_sh030_anim.mov
  CLIP: test_title
  CLIP: sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
""", out)

    def test_audio_only(self):
        sys.argv = [
            'otiotool',
            '-i', PREMIERE_EXAMPLE_PATH,
            '--audio-only',
            '--list-clips'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: sc01_sh010_layerA
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_placeholder.wav
  CLIP: track_08.wav
  CLIP: sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
""", out)

    def test_only_tracks_with_name(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--only-tracks-with-name', 'Sequence 3',
            '--list-clips'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: OTIO TEST - multitrack.Exported.01
  CLIP: KOLL-HD.mp4
""", out)

    def test_only_tracks_with_index(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--only-tracks-with-index', '3',
            '--list-clips'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: OTIO TEST - multitrack.Exported.01
  CLIP: KOLL-HD.mp4
""", out)

    def test_only_tracks_with_index2(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--only-tracks-with-index', '2', '3',
            '--list-clips'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: OTIO TEST - multitrack.Exported.01
  CLIP: t-hawk (loop)-HD.mp4
  CLIP: KOLL-HD.mp4
""", out)

    def test_only_clips_with_name(self):
        sys.argv = [
            'otiotool',
            '-i', PREMIERE_EXAMPLE_PATH,
            '--list-clips',
            '--list-tracks',
            '--only-clips-with-name', 'sc01_sh010_anim.mov'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: sc01_sh010_layerA
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
TRACK:  (Video)
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
TRACK:  (Audio)
TRACK:  (Audio)
  CLIP: sc01_sh010_anim.mov
""", out)

    def test_only_clips_with_regex(self):
        sys.argv = [
            'otiotool',
            '-i', PREMIERE_EXAMPLE_PATH,
            '--list-clips',
            '--list-tracks',
            '--only-clips-with-name-regex', 'anim'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: sc01_sh010_layerA
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh020_anim.mov
  CLIP: sc01_sh030_anim.mov
TRACK:  (Video)
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
TRACK:  (Audio)
TRACK:  (Audio)
  CLIP: sc01_sh010_anim.mov
""", out)

    def test_remote_transition(self):
        sys.argv = [
            'otiotool',
            '-i', TRANSITION_PATH,
            '-o', '-',
            '--remove-transitions'
        ]
        out, err = self.run_test()
        self.assertNotIn('"OTIO_SCHEMA": "Transition.', out)

    def test_trim(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--trim', '20', '40',
            '--list-clips',
            '--inspect', 't-hawk'
        ]
        out, err = self.run_test()
        self.assertEqual(
            ("TIMELINE: OTIO TEST - multitrack.Exported.01\n"
             "  ITEM: t-hawk (loop)-HD.mp4 (<class 'opentimelineio._otio.Clip'>)\n"  # noqa E501 line too long
             "    source_range: TimeRange(RationalTime(0, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "    trimmed_range: TimeRange(RationalTime(0, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "    visible_range: TimeRange(RationalTime(0, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "    range_in_parent: TimeRange(RationalTime(2, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "    trimmed range in timeline: TimeRange(RationalTime(2, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "    visible range in timeline: TimeRange(RationalTime(2, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "    range in Sequence 2 (<class 'opentimelineio._otio.Track'>): TimeRange(RationalTime(2, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "    range in NestedScope (<class 'opentimelineio._otio.Stack'>): TimeRange(RationalTime(2, 24), RationalTime(478, 24))\n"  # noqa E501 line too long
             "TIMELINE: OTIO TEST - multitrack.Exported.01\n"
             "  CLIP: tech.fux (loop)-HD.mp4\n"
             "  CLIP: out-b (loop)-HD.mp4\n"
             "  CLIP: t-hawk (loop)-HD.mp4\n"),
            out)

    def test_flatten(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--flatten', 'video',
            '--list-clips',
            '--list-tracks',
            '--inspect', 'out-b'
        ]
        out, err = self.run_test()
        self.assertEqual(
            ("TIMELINE: OTIO TEST - multitrack.Exported.01\n"
             "  ITEM: out-b (loop)-HD.mp4 (<class 'opentimelineio._otio.Clip'>)\n"  # noqa E501 line too long
             "    source_range: TimeRange(RationalTime(159, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    trimmed_range: TimeRange(RationalTime(159, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    visible_range: TimeRange(RationalTime(159, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    range_in_parent: TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    trimmed range in timeline: TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    visible range in timeline: TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    range in Flattened (<class 'opentimelineio._otio.Track'>): TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    range in NestedScope (<class 'opentimelineio._otio.Stack'>): TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "TIMELINE: OTIO TEST - multitrack.Exported.01\n"
             "TRACK: Flattened (Video)\n"
             "  CLIP: tech.fux (loop)-HD.mp4\n"
             "  CLIP: t-hawk (loop)-HD.mp4\n"
             "  CLIP: out-b (loop)-HD.mp4\n"
             "  CLIP: KOLL-HD.mp4\n"
             "  CLIP: brokchrd (loop)-HD.mp4\n"),
            out)

    def test_keep_flattened_tracks(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--flatten', 'video',
            '--keep-flattened-tracks',
            '--list-clips',
            '--list-tracks',
            '--inspect', 'out-b'
        ]
        out, err = self.run_test()
        self.assertEqual(
            ("TIMELINE: OTIO TEST - multitrack.Exported.01\n"
             "  ITEM: out-b (loop)-HD.mp4 (<class 'opentimelineio._otio.Clip'>)\n"  # noqa E501 line too long
             "    source_range: TimeRange(RationalTime(0, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "    trimmed_range: TimeRange(RationalTime(0, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "    visible_range: TimeRange(RationalTime(0, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "    range_in_parent: TimeRange(RationalTime(803, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "    trimmed range in timeline: TimeRange(RationalTime(803, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "    visible range in timeline: TimeRange(RationalTime(803, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "    range in Sequence (<class 'opentimelineio._otio.Track'>): TimeRange(RationalTime(803, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "    range in NestedScope (<class 'opentimelineio._otio.Stack'>): TimeRange(RationalTime(803, 24), RationalTime(722, 24))\n"  # noqa E501 line too long
             "  ITEM: out-b (loop)-HD.mp4 (<class 'opentimelineio._otio.Clip'>)\n"  # noqa E501 line too long
             "    source_range: TimeRange(RationalTime(159, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    trimmed_range: TimeRange(RationalTime(159, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    visible_range: TimeRange(RationalTime(159, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    range_in_parent: TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    trimmed range in timeline: TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    visible range in timeline: TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    range in Flattened (<class 'opentimelineio._otio.Track'>): TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "    range in NestedScope (<class 'opentimelineio._otio.Stack'>): TimeRange(RationalTime(962, 24), RationalTime(236, 24))\n"  # noqa E501 line too long
             "TIMELINE: OTIO TEST - multitrack.Exported.01\n"
             "TRACK: Sequence (Video)\n"
             "  CLIP: tech.fux (loop)-HD.mp4\n"
             "  CLIP: out-b (loop)-HD.mp4\n"
             "  CLIP: brokchrd (loop)-HD.mp4\n"
             "TRACK: Sequence 2 (Video)\n"
             "  CLIP: t-hawk (loop)-HD.mp4\n"
             "TRACK: Sequence 3 (Video)\n"
             "  CLIP: KOLL-HD.mp4\n"
             "TRACK: Flattened (Video)\n"
             "  CLIP: tech.fux (loop)-HD.mp4\n"
             "  CLIP: t-hawk (loop)-HD.mp4\n"
             "  CLIP: out-b (loop)-HD.mp4\n"
             "  CLIP: KOLL-HD.mp4\n"
             "  CLIP: brokchrd (loop)-HD.mp4\n"),
            out)

    def test_stack(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH, PREMIERE_EXAMPLE_PATH,
            '--stack',
            '--list-clips',
            '--list-tracks',
            '--stats'
        ]
        out, err = self.run_test()
        self.maxDiff = None
        self.assertEqual("""Name: Stacked 2 Timelines
Start:    00:00:00:00
End:      00:02:16:18
Duration: 00:02:16:18
TIMELINE: Stacked 2 Timelines
TRACK: Sequence (Video)
  CLIP: tech.fux (loop)-HD.mp4
  CLIP: out-b (loop)-HD.mp4
  CLIP: brokchrd (loop)-HD.mp4
TRACK: Sequence 2 (Video)
  CLIP: t-hawk (loop)-HD.mp4
TRACK: Sequence 3 (Video)
  CLIP: KOLL-HD.mp4
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh020_anim.mov
  CLIP: sc01_sh030_anim.mov
TRACK:  (Video)
  CLIP: test_title
TRACK:  (Video)
  CLIP: sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_placeholder.wav
TRACK:  (Audio)
  CLIP: track_08.wav
TRACK:  (Audio)
  CLIP: sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
""", out)

    def test_concat(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH, PREMIERE_EXAMPLE_PATH,
            '--concat',
            '--list-clips',
            '--list-tracks',
            '--stats'
        ]
        out, err = self.run_test()
        self.maxDiff = None
        self.assertEqual("""Name: Concatenated 2 Timelines
Start:    00:00:00:00
End:      00:02:59:03
Duration: 00:02:59:03
TIMELINE: Concatenated 2 Timelines
TRACK:  (Video)
TRACK: Sequence (Video)
  CLIP: tech.fux (loop)-HD.mp4
  CLIP: out-b (loop)-HD.mp4
  CLIP: brokchrd (loop)-HD.mp4
TRACK: Sequence 2 (Video)
  CLIP: t-hawk (loop)-HD.mp4
TRACK: Sequence 3 (Video)
  CLIP: KOLL-HD.mp4
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
TRACK:  (Video)
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh020_anim.mov
  CLIP: sc01_sh030_anim.mov
TRACK:  (Video)
  CLIP: test_title
TRACK:  (Video)
  CLIP: sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_sh010_anim.mov
  CLIP: sc01_sh010_anim.mov
TRACK:  (Audio)
  CLIP: sc01_placeholder.wav
TRACK:  (Audio)
  CLIP: track_08.wav
TRACK:  (Audio)
  CLIP: sc01_master_layerA_sh030_temp.mov
  CLIP: sc01_sh010_anim.mov
""", out)

    def test_redact(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--redact',
            '--list-clips',
            '--list-tracks'
        ]
        out, err = self.run_test()
        self.assertEqual("""TIMELINE: Timeline #1
TRACK: Track #1 (Video)
  CLIP: Clip #1
  CLIP: Clip #2
  CLIP: Clip #3
TRACK: Track #2 (Video)
  CLIP: Clip #4
TRACK: Track #3 (Video)
  CLIP: Clip #5
""", out)

    def test_stats(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--stats'
        ]
        out, err = self.run_test()
        self.assertEqual("""Name: OTIO TEST - multitrack.Exported.01
Start:    00:00:00:00
End:      00:02:16:18
Duration: 00:02:16:18
""", out)

    def test_inspect(self):
        sys.argv = [
            'otiotool',
            '-i', MULTITRACK_PATH,
            '--inspect', 'KOLL'
        ]
        out, err = self.run_test()
        self.assertEqual(
            ("TIMELINE: OTIO TEST - multitrack.Exported.01\n"
             "  ITEM: KOLL-HD.mp4 (<class 'opentimelineio._otio.Clip'>)\n"
             "    source_range: TimeRange(RationalTime(0, 24), RationalTime(640, 24))\n"  # noqa E501 line too long
             "    trimmed_range: TimeRange(RationalTime(0, 24), RationalTime(640, 24))\n"  # noqa E501 line too long
             "    visible_range: TimeRange(RationalTime(0, 24), RationalTime(640, 24))\n"  # noqa E501 line too long
             "    range_in_parent: TimeRange(RationalTime(1198, 24), RationalTime(640, 24))\n"  # noqa E501 line too long
             "    trimmed range in timeline: TimeRange(RationalTime(1198, 24), RationalTime(640, 24))\n"  # noqa E501 line too long
             "    visible range in timeline: TimeRange(RationalTime(1198, 24), RationalTime(640, 24))\n"  # noqa E501 line too long
             "    range in Sequence 3 (<class 'opentimelineio._otio.Track'>): TimeRange(RationalTime(1198, 24), RationalTime(640, 24))\n"  # noqa E501 line too long
             "    range in NestedScope (<class 'opentimelineio._otio.Stack'>): TimeRange(RationalTime(1198, 24), RationalTime(640, 24))\n"),  # noqa E501 line too long
            out)


OTIOToolTest_ShellOut = CreateShelloutTest(OTIOToolTest)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""Example OTIO script that reads a timeline and then prints a summary
of the video clips found, including re-timing effects on each one.
"""

import sys
import opentimelineio as otio


def _summarize_effects(item):
    if hasattr(item, "effects"):
        for effect in item.effects:
            if isinstance(effect, otio.schema.FreezeFrame):
                print(
                    "Effect: Freeze Frame"
                )
            elif isinstance(effect, otio.schema.LinearTimeWarp):
                print(
                    "Effect: Linear Time Warp ({}%)".format(
                        effect.time_scalar * 100.0
                    )
                )
            else:
                print(
                    "Effect: {} ({})".format(
                        effect.name,
                        str(type(effect))
                    )
                )


def _summarize_range(label, time_range):
    if time_range is None:
        print("\t{}: None".format(label))
    else:
        print(
            "\t{}: {} - {} (Duration: {})".format(
                label,
                otio.opentime.to_timecode(
                    time_range.start_time
                ),
                otio.opentime.to_timecode(
                    time_range.end_time_exclusive()
                ),
                otio.opentime.to_timecode(time_range.duration)
            )
        )


def _summarize_timeline(timeline):
    # Here we iterate over each video track, and then just the top-level
    # items in each track. If you want to traverse the whole nested structure
    # then you can use: for item in timeline.each_child()
    # or just clips via: for clip in timeline.each_clip()
    # See also: https://opentimelineio.readthedocs.io/en/latest/tutorials/otio-timeline-structure.html  # noqa
    for track in timeline.video_tracks():
        print(
            "Track: {}\n\tKind: {}\n\tDuration: {}".format(
                track.name,
                track.kind,
                track.duration()
            )
        )
        _summarize_effects(track)

        for item in track:
            if isinstance(item, otio.schema.Clip):
                clip = item
                print("Clip: {}".format(clip.name))
                # See the documentation to understand the difference
                # between each of these ranges:
                # https://opentimelineio.readthedocs.io/en/latest/tutorials/time-ranges.html
                _summarize_range("  Trimmed Range", clip.trimmed_range())
                _summarize_range("  Visible Range", clip.visible_range())
                _summarize_range("Available Range", clip.available_range())

            elif isinstance(item, otio.schema.Gap):
                continue

            elif isinstance(item, otio.schema.Transition):
                transition = item
                print(
                    "Transition: {}\n\tDuration: {}".format(
                        transition.transition_type,
                        otio.opentime.to_timecode(transition.duration())
                    )
                )

            elif isinstance(item, otio.core.Composition):
                composition = item
                print(
                    "Nested Composition: {}\n\tDuration: {}".format(
                        composition.name,
                        otio.opentime.to_timecode(composition.duration())
                    )
                )

            else:
                print(
                    "Other: {} ({})\n\tDuration: {}".format(
                        item.name,
                        str(type(item)),
                        otio.opentime.to_timecode(item.duration())
                    )
                )

            _summarize_effects(item)


def main():
    for filename in sys.argv[1:]:

        timeline = otio.adapters.read_from_file(filename)
        _summarize_timeline(timeline)


if __name__ == '__main__':
    main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

""" Unit tests of functions that convert between file paths and urls. """

import unittest
import os

import opentimelineio as otio

SAMPLE_DATA_DIR = os.path.join(os.path.dirname(__file__), "sample_data")
SCREENING_EXAMPLE_PATH = os.path.join(SAMPLE_DATA_DIR, "screening_example.edl")
MEDIA_EXAMPLE_PATH_REL = os.path.relpath(
    os.path.join(
        os.path.dirname(__file__),
        "..",  # root
        "docs",
        "_static",
        "OpenTimelineIO@3xDark.png"
    )
)
MEDIA_EXAMPLE_PATH_URL_REL = otio.url_utils.url_from_filepath(
    MEDIA_EXAMPLE_PATH_REL
)
MEDIA_EXAMPLE_PATH_ABS = os.path.abspath(
    MEDIA_EXAMPLE_PATH_REL.replace(
        "3xDark",
        "3xLight"
    )
)
MEDIA_EXAMPLE_PATH_URL_ABS = otio.url_utils.url_from_filepath(
    MEDIA_EXAMPLE_PATH_ABS
)


class TestConversions(unittest.TestCase):
    def test_roundtrip_abs(self):
        self.assertTrue(MEDIA_EXAMPLE_PATH_URL_ABS.startswith("file://"))
        full_path = os.path.abspath(
            otio.url_utils.filepath_from_url(MEDIA_EXAMPLE_PATH_URL_ABS)
        )

        # should have reconstructed it by this point
        self.assertEqual(full_path, MEDIA_EXAMPLE_PATH_ABS)

    def test_roundtrip_rel(self):
        self.assertFalse(MEDIA_EXAMPLE_PATH_URL_REL.startswith("file://"))

        result = otio.url_utils.filepath_from_url(MEDIA_EXAMPLE_PATH_URL_REL)

        # should have reconstructed it by this point
        self.assertEqual(os.path.normpath(result), MEDIA_EXAMPLE_PATH_REL)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
#
# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""Example OTIO script that reads a timeline and then relinks clips
to movie files found in a given folder, based on matching clip names to filenames.

Demo:

% ls -1R
editorial_cut.otio
media/
    shot1.mov
    shot17.mov
    shot99.mov

% conform.py -i editorial_cut.otio -f media -o conformed.otio
Relinked 3 clips to new media.
Saved conformed.otio with 100 clips.

% diff editorial_cut.otio conformed.otio
...

"""

import argparse
from argparse import RawTextHelpFormatter
import glob
import os

import opentimelineio as otio


def parse_args():
    """ parse arguments out of sys.argv """
    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=RawTextHelpFormatter)
    parser.add_argument(
        '-i',
        '--input',
        type=str,
        required=True,
        help='Timeline file to read. Supported formats: {adapters}'
             ''.format(adapters=otio.adapters.available_adapter_names())
    )
    parser.add_argument(
        '-f',
        '--folder',
        type=str,
        required=True,
        help='Folder to look for media in.'
    )
    parser.add_argument(
        '-o',
        '--output',
        type=str,
        required=True,
        help="Timeline file to write out."
    )
    return parser.parse_args()


def _find_matching_media(name, folder):
    """Look for media with this name in this folder."""

    # This function is an example which searches the file system for matching media.
    # A real world studio implementation would likely look in an asset management system
    # and use studio-specific metadata in the clip's metadata dictionary instead
    # of matching the clip name.
    # For example:
    # shot = asset_database.find_shot(clip.metadata['mystudio']['shotID'])
    # new_media = shot.latest_render(format='mov')

    matches = glob.glob("{0}/{1}.*".format(folder, name))
    matches = list(map(os.path.abspath, matches))

    if not matches:
        # print "DEBUG: No match for clip '{0}'".format(name)
        return None
    if len(matches) == 1:
        return matches[0]
    else:
        print(
            "WARNING: {0} matches found for clip '{1}', using '{2}'".format(
                len(matches),
                name,
                matches[0]
            )
        )
        return matches[0]


def _conform_timeline(timeline, folder):
    """ Look for replacement media for each clip in the given timeline.

    The clips are relinked in place if media with a matching name is found.
    """

    count = 0

    for clip in timeline.each_clip():
        # look for a media file that matches the clip's name
        new_path = _find_matching_media(clip.name, folder)

        # if no media is found, keep going
        if not new_path:
            continue

        # relink to the found path
        clip.media_reference = otio.schema.ExternalReference(
            target_url="file://" + new_path,
            available_range=None  # the available range is unknown without
                                  # opening the file
        )
        count += 1

    return count


def main():
    args = parse_args()

    timeline = otio.adapters.read_from_file(args.input)
    count = _conform_timeline(timeline, args.folder)
    print("Relinked {0} clips to new media.".format(count))
    otio.adapters.write_to_file(timeline, args.output)
    print(
        "Saved {} with {} clips.".format(
            args.output,
            len(list(timeline.each_clip()))
        )
    )


if __name__ == '__main__':
    main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""Unit tests for the rv session file adapter"""

import os
import tempfile
import unittest

import opentimelineio as otio

SAMPLE_DATA_DIR = os.path.join(os.path.dirname(__file__), "sample_data")
SCREENING_EXAMPLE_PATH = os.path.join(SAMPLE_DATA_DIR, "screening_example.edl")
TRANSITION_EXAMPLE_PATH = os.path.join(SAMPLE_DATA_DIR, "transition_test.otio")
BASELINE_PATH = os.path.join(SAMPLE_DATA_DIR, "screening_example.rv")
BASELINE_TRANSITION_PATH = os.path.join(SAMPLE_DATA_DIR, "transition_test.rv")
METADATA_EXAMPLE_PATH = os.path.join(SAMPLE_DATA_DIR, "rv_metadata.otio")
METADATA_BASELINE_PATH = os.path.join(SAMPLE_DATA_DIR, "rv_metadata.rv")
IMAGE_SEQUENCE_EXAMPLE_PATH = os.path.join(
    SAMPLE_DATA_DIR,
    "image_sequence_example.otio"
)


SAMPLE_DATA = """{
    "OTIO_SCHEMA": "Timeline.1",
    "tracks": {
        "OTIO_SCHEMA": "Stack.1",
        "children": [{
            "OTIO_SCHEMA": "Track.1",
            "kind": "Video",
            "children": [{
                "OTIO_SCHEMA": "Gap.1",
                "source_range": {
                    "OTIO_SCHEMA": "TimeRange.1",
                    "duration": {
                    "OTIO_SCHEMA": "RationalTime.1",
                    "rate": 24.0, "value": 10.0
                },
                "start_time": {
                    "OTIO_SCHEMA": "RationalTime.1",
                    "rate": 24.0, "value": 0.0
                }
                }
            },
            {
                "OTIO_SCHEMA": "Transition.1",
                "transition_type": "SMPTE_Dissolve",
                "in_offset": {
                    "OTIO_SCHEMA": "RationalTime.1",
                    "rate": 24.0, "value": 10.0
                },
                "out_offset": {
                    "OTIO_SCHEMA": "RationalTime.1",
                    "rate": 24.0, "value": 10.0
                }
            },
            {
                "OTIO_SCHEMA": "Clip.1",
                "media_reference": {
                    "OTIO_SCHEMA": "MissingReference.1"
                },
                "source_range": {
                    "OTIO_SCHEMA": "TimeRange.1",
                    "duration": {
                        "OTIO_SCHEMA": "RationalTime.1",
                        "rate": 24.0, "value": 10.0
                    },
                    "start_time": {
                        "OTIO_SCHEMA": "RationalTime.1",
                        "rate": 24.0, "value": 10.0
                    }
                }
            }]
        }]
    }
}"""


AUDIO_VIDEO_SAMPLE_DATA = """{
    "OTIO_SCHEMA": "Timeline.1",
    "metadata": {},
    "name": null,
    "tracks": {
        "OTIO_SCHEMA": "Stack.1",
        "children": [
            {
                "OTIO_SCHEMA": "Track.1",
                "children": [
                    {
                        "OTIO_SCHEMA": "Clip.1",
                        "effects": [],
                        "markers": [],
                        "media_reference": {
                            "OTIO_SCHEMA": "ExternalReference.1",
                            "available_range": {
                                "OTIO_SCHEMA": "TimeRange.1",
                                "duration": {
                                    "OTIO_SCHEMA": "RationalTime.1",
                                    "rate": 25,
                                    "value": 67
                                },
                                "start_time": {
                                    "OTIO_SCHEMA": "RationalTime.1",
                                    "rate": 25,
                                    "value": 0
                                }
                            },
                            "metadata": {},
                            "name": null,
                            "target_url": "/path/to/video.mov"
                        },
                        "metadata": {},
                        "name": "plyblast",
                        "source_range": {
                            "OTIO_SCHEMA": "TimeRange.1",
                            "duration": {
                                "OTIO_SCHEMA": "RationalTime.1",
                                "rate": 25,
                                "value": 67
                            },
                            "start_time": {
                                "OTIO_SCHEMA": "RationalTime.1",
                                "rate": 25,
                                "value": 54
                            }
                        }
                    }
                ],
                "effects": [],
                "kind": "Video",
                "markers": [],
                "metadata": {},
                "name": "v1",
                "source_range": null
            },
            {
                "OTIO_SCHEMA": "Track.1",
                "children": [
                    {
                        "OTIO_SCHEMA": "Clip.1",
                        "effects": [],
                        "markers": [],
                        "media_reference": {
                            "OTIO_SCHEMA": "ExternalReference.1",
                            "available_range": {
                                "OTIO_SCHEMA": "TimeRange.1",
                                "duration": {
                                    "OTIO_SCHEMA": "RationalTime.1",
                                    "rate": 25,
                                    "value": 500
                                },
                                "start_time": {
                                    "OTIO_SCHEMA": "RationalTime.1",
                                    "rate": 25,
                                    "value": 0
                                }
                            },
                            "metadata": {},
                            "name": null,
                            "target_url": "/path/to/audio.wav"
                        },
                        "metadata": {},
                        "name": "sound",
                        "source_range": {
                            "OTIO_SCHEMA": "TimeRange.1",
                            "duration": {
                                "OTIO_SCHEMA": "RationalTime.1",
                                "rate": 25,
                                "value": 67
                            },
                            "start_time": {
                                "OTIO_SCHEMA": "RationalTime.1",
                                "rate": 25,
                                "value": 54
                            }
                        }
                    }
                ],
                "effects": [],
                "kind": "Audio",
                "markers": [],
                "metadata": {},
                "name": "a1",
                "source_range": null
            }
        ],
        "effects": [],
        "markers": [],
        "metadata": {},
        "name": "tracks",
        "source_range": null
    }
}"""


NESTED_STACK_SAMPLE_DATA = """{
    "OTIO_SCHEMA": "Timeline.1",
    "metadata": {},
    "name": "My Timeline",
    "tracks": {
        "OTIO_SCHEMA": "Stack.1",
        "children": [
            {
                "OTIO_SCHEMA": "Track.1",
                "children": [
                    {
                        "OTIO_SCHEMA": "Clip.1",
                        "effects": [],
                        "markers": [],
                        "media_reference": {
                            "OTIO_SCHEMA": "ExternalReference.1",
                            "available_range": {
                                "OTIO_SCHEMA": "TimeRange.1",
                                "duration": {
                                    "OTIO_SCHEMA": "RationalTime.1",
                                    "rate": 24,
                                    "value": 238
                                },
                                "start_time": {
                                    "OTIO_SCHEMA": "RationalTime.1",
                                    "rate": 24,
                                    "value": 0
                                }
                            },
                            "metadata": {},
                            "target_url": "/path/to/some/video.mov"
                        },
                        "metadata": {},
                        "name": "Normal Clip 1",
                        "source_range": {
                            "OTIO_SCHEMA": "TimeRange.1",
                            "duration": {
                                "OTIO_SCHEMA": "RationalTime.1",
                                "rate": 24,
                                "value": 238
                            },
                            "start_time": {
                                "OTIO_SCHEMA": "RationalTime.1",
                                "rate": 24,
                                "value": 135
                            }
                        }
                    },
                    {
                        "OTIO_SCHEMA": "Stack.1",
                        "children": [
                            {
                                "OTIO_SCHEMA": "Clip.1",
                                "effects": [],
                                "markers": [],
                                "media_reference": {
                                    "OTIO_SCHEMA": "ExternalReference.1",
                                    "available_range": {
                                        "OTIO_SCHEMA": "TimeRange.1",
                                        "duration": {
                                            "OTIO_SCHEMA": "RationalTime.1",
                                            "rate": 24,
                                            "value": 238
                                        },
                                        "start_time": {
                                            "OTIO_SCHEMA": "RationalTime.1",
                                            "rate": 24,
                                            "value": 0
                                        }
                                    },
                                    "metadata": {},
                                    "target_url": "/path/to/some/video.mov"
                                },
                                "metadata": {},
                                "name": "Clip Inside A Stack 1",
                                "source_range": {
                                    "OTIO_SCHEMA": "TimeRange.1",
                                    "duration": {
                                        "OTIO_SCHEMA": "RationalTime.1",
                                        "rate": 24,
                                        "value": 37
                                    },
                                    "start_time": {
                                        "OTIO_SCHEMA": "RationalTime.1",
                                        "rate": 24,
                                        "value": 373
                                    }
                                }
                            }
                        ],
                        "effects": [],
                        "markers": [],
                        "metadata": {},
                        "name": "Nested Stack 1",
                        "source_range": {
                            "OTIO_SCHEMA": "TimeRange.1",
                            "duration": {
                                "OTIO_SCHEMA": "RationalTime.1",
                                "rate": 24,
                                "value": 31
                            },
                            "start_time": {
                                "OTIO_SCHEMA": "RationalTime.1",
                                "rate": 24,
                                "value": 0
                            }
                        }
                    }
                ],
                "effects": [],
                "kind": "Video",
                "markers": [],
                "metadata": {},
                "name": "Top Level Track",
                "source_range": null
            },
            {
                "OTIO_SCHEMA": "Track.1",
                "children": [
                    {
                        "OTIO_SCHEMA": "Clip.1",
                        "effects": [],
                        "markers": [],
                        "media_reference": {
                            "OTIO_SCHEMA": "ExternalReference.1",
                            "available_range": {
                                "OTIO_SCHEMA": "TimeRange.1",
                                "duration": {
                                    "OTIO_SCHEMA": "RationalTime.1",
                                    "rate": 24,
                                    "value": 238
                                },
                                "start_time": {
                                    "OTIO_SCHEMA": "RationalTime.1",
                                    "rate": 24,
                                    "value": 0
                                }
                            },
                            "metadata": {},
                            "target_url": "/path/to/some/audio.wav"
                        },
                        "metadata": {},
                        "name": "Normal Clip 1",
                        "source_range": {
                            "OTIO_SCHEMA": "TimeRange.1",
                            "duration": {
                                "OTIO_SCHEMA": "RationalTime.1",
                                "rate": 24,
                                "value": 238
                            },
                            "start_time": {
                                "OTIO_SCHEMA": "RationalTime.1",
                                "rate": 24,
                                "value": 135
                            }
                        }
                    },
                    {
                        "OTIO_SCHEMA": "Stack.1",
                        "children": [
                            {
                                "OTIO_SCHEMA": "Clip.1",
                                "effects": [],
                                "markers": [],
                                "media_reference": {
                                    "OTIO_SCHEMA": "ExternalReference.1",
                                    "available_range": {
                                        "OTIO_SCHEMA": "TimeRange.1",
                                        "duration": {
                                            "OTIO_SCHEMA": "RationalTime.1",
                                            "rate": 24,
                                            "value": 238
                                        },
                                        "start_time": {
                                            "OTIO_SCHEMA": "RationalTime.1",
                                            "rate": 24,
                                            "value": 0
                                        }
                                    },
                                    "metadata": {},
                                    "target_url": "/path/to/some/audio.wav"
                                },
                                "metadata": {},
                                "name": "Clip Inside A Stack 1",
                                "source_range": {
                                    "OTIO_SCHEMA": "TimeRange.1",
                                    "duration": {
                                        "OTIO_SCHEMA": "RationalTime.1",
                                        "rate": 24,
                                        "value": 37
                                    },
                                    "start_time": {
                                        "OTIO_SCHEMA": "RationalTime.1",
                                        "rate": 24,
                                        "value": 373
                                    }
                                }
                            }
                        ],
                        "effects": [],
                        "markers": [],
                        "metadata": {},
                        "name": "Nested Stack 1",
                        "source_range": {
                            "OTIO_SCHEMA": "TimeRange.1",
                            "duration": {
                                "OTIO_SCHEMA": "RationalTime.1",
                                "rate": 24,
                                "value": 31
                            },
                            "start_time": {
                                "OTIO_SCHEMA": "RationalTime.1",
                                "rate": 24,
                                "value": 0
                            }
                        }
                    }
                ],
                "effects": [],
                "kind": "Audio",
                "markers": [],
                "metadata": {},
                "name": "Top Level Track",
                "source_range": null
            }
        ],
        "effects": [],
        "markers": [],
        "metadata": {},
        "name": "Top Level Stack",
        "source_range": null
    }
}"""


@unittest.skipIf(
    "OTIO_RV_PYTHON_LIB" not in os.environ or
    "OTIO_RV_PYTHON_BIN" not in os.environ,
    "OTIO_RV_PYTHON_BIN or OTIO_RV_PYTHON_LIB not set."
)
class RVSessionAdapterReadTest(unittest.TestCase):
    def setUp(self):
        fd, self.tmp_path = tempfile.mkstemp(suffix=".rv", text=True)

        # Close file descriptor to avoid leak. We only need the tmp_path.
        os.close(fd)

    def tearDown(self):
        os.unlink(self.tmp_path)

    def test_basic_rvsession_read(self):
        timeline = otio.adapters.read_from_file(SCREENING_EXAMPLE_PATH)

        otio.adapters.write_to_file(timeline, self.tmp_path)

        with open(self.tmp_path) as fo:
            test_data = fo.read()

        with open(BASELINE_PATH) as fo:
            baseline_data = fo.read()

        self.maxDiff = None
        self._connectionFreeAssertMultiLineEqual(baseline_data, test_data)

    def test_transition_rvsession_read(self):
        timeline = otio.adapters.read_from_file(TRANSITION_EXAMPLE_PATH)

        otio.adapters.write_to_file(timeline, self.tmp_path)
        self.assertTrue(os.path.exists(self.tmp_path))

        with open(self.tmp_path) as fo:
            test_data = fo.read()

        with open(BASELINE_TRANSITION_PATH) as fo:
            baseline_data = fo.read()

        self.maxDiff = None
        self._connectionFreeAssertMultiLineEqual(baseline_data, test_data)

    def test_image_sequence_example(self):
        # SETUP
        timeline = otio.adapters.read_from_file(IMAGE_SEQUENCE_EXAMPLE_PATH)

        # EXERCISE
        otio.adapters.write_to_file(timeline, self.tmp_path)

        # VERIFY
        self.assertTrue(os.path.exists(self.tmp_path))

        with open(self.tmp_path) as f:
            rv_session = f.read()

        self.assertEqual(
            rv_session.count(
                'string movie = "./sample_sequence/sample_sequence.%04d.exr"'
            ),
            1
        )

    def test_transition_rvsession_covers_entire_shots(self):
        # SETUP
        timeline = otio.adapters.read_from_string(SAMPLE_DATA, "otio_json")

        # EXERCISE
        otio.adapters.write_to_file(timeline, self.tmp_path)

        # VERIFY
        with open(self.tmp_path, "r") as f:
            rv_session = f.read()

        self.assertEqual(rv_session.count('movie = "blank'), 1)
        self.assertEqual(rv_session.count('movie = "smpte'), 1)

    def test_audio_video_tracks(self):
        # SETUP
        timeline = otio.adapters.read_from_string(AUDIO_VIDEO_SAMPLE_DATA, "otio_json")

        # EXERCISE
        otio.adapters.write_to_file(timeline, self.tmp_path)

        # VERIFY
        self.assertTrue(os.path.exists(self.tmp_path))

        audio_video_source = (
            'string movie = '
            '[ "blank,start=0.0,end=499.0,fps=25.0.movieproc" '
            '"blank,start=0.0,end=499.0,fps=25.0.movieproc"'
            ' "/path/to/audio.wav" ]'
        )

        with open(self.tmp_path, "r") as f:
            rv_session = f.read()

        self.assertEqual(rv_session.count("string movie"), 2)
        self.assertEqual(rv_session.count("blank"), 2)
        self.assertEqual(rv_session.count(audio_video_source), 1)

    def test_nested_stack(self):
        # SETUP
        timeline = otio.adapters.read_from_string(
            NESTED_STACK_SAMPLE_DATA,
            "otio_json"
        )

        # EXERCISE
        otio.adapters.write_to_file(timeline, self.tmp_path)

        # VERIFY
        self.assertTrue(os.path.exists(self.tmp_path))

        audio_video_source = (
            'string movie = '
            '[ "blank,start=0.0,end=237.0,fps=24.0.movieproc"'
            ' "blank,start=0.0,end=237.0,fps=24.0.movieproc"'
            ' "/path/to/some/audio.wav" ]'
        )
        video_source = (
            'string movie = "/path/to/some/video.mov"'
        )

        with open(self.tmp_path, "r") as f:
            rv_session = f.read()
            self.assertEqual(rv_session.count(video_source), 2)
            self.assertEqual(rv_session.count(audio_video_source), 2)

    def test_metadata_read(self):
        timeline = otio.adapters.read_from_file(METADATA_EXAMPLE_PATH)
        tmp_path = tempfile.mkstemp(suffix=".rv", text=True)[1]

        otio.adapters.write_to_file(timeline, tmp_path)
        self.assertTrue(os.path.exists(tmp_path))

        with open(tmp_path) as fo:
            test_data = fo.read()

        with open(METADATA_BASELINE_PATH) as fo:
            baseline_data = fo.read()

        self.maxDiff = None
        self._connectionFreeAssertMultiLineEqual(baseline_data, test_data)

    def _connectionFreeAssertMultiLineEqual(self, first, second):
        '''
        The "connections" list order is not stable between python versions
        so as a quick hack, simply remove it from our diff
        '''
        def _removeConnections(string):
            return os.linesep.join([line for line in string.splitlines()
                                    if 'connections' not in line])
        self.assertMultiLineEqual(_removeConnections(first),
                                  _removeConnections(second))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""Test harness for the filter algorithms."""

import unittest
import copy

import opentimelineio as otio
import opentimelineio.test_utils as otio_test_utils


class FilterTest(unittest.TestCase, otio_test_utils.OTIOAssertions):
    maxDiff = None

    def test_copy_track(self):
        md = {'test': 'bar'}
        tr = otio.schema.Track(name='foo', metadata=md)
        tr.append(otio.schema.Clip(name='cl1', metadata=md))

        test = otio.algorithms.filtered_composition(tr, lambda _: _)

        self.assertJsonEqual(tr, test)

    def test_copy_stack(self):
        """Test a no op filter that copies the timeline."""

        md = {'test': 'bar'}
        tr = otio.schema.Stack(name='foo', metadata=md)
        tr.append(otio.schema.Clip(name='cl1', metadata=md))

        result = otio.algorithms.filtered_composition(tr, lambda _: _)

        self.assertJsonEqual(tr, result)
        self.assertIsNot(tr[0], result)

    def test_prune_clips_starting_with_a(self):
        """test a filter that removes things whose name starts with 'a'"""

        md = {'test': 'bar'}
        tr = otio.schema.Track(name='foo', metadata=md)
        tr.append(otio.schema.Clip(name='cl1', metadata=md))
        tr.append(otio.schema.Clip(name='a_cl3', metadata=md))
        tr_2 = otio.schema.Track(name='a', metadata=md)
        tr_2.append(otio.schema.Clip(name='cl1', metadata=md))
        tr_2.append(otio.schema.Clip(name='a_cl2', metadata=md))
        tr.append(tr_2)

        visited = []

        def nothing_that_starts_with_a(thing):
            visited.append(thing.name)
            if thing.name.startswith('a'):
                return None
            else:
                return thing

        result = otio.algorithms.filtered_composition(
            tr,
            nothing_that_starts_with_a
        )

        # make sure that the track being pruned means the child was never
        # visited
        self.assertNotIn('a_cl2', visited)

        # match the desired behavior of the function
        del tr[2]
        del tr[1]

        self.assertJsonEqual(tr, result)

    def test_prune_clips(self):
        """test a filter that removes clips"""

        md = {'test': 'bar'}
        tr = otio.schema.Track(name='foo', metadata=md)
        tr.append(otio.schema.Clip(name='cl1', metadata=md))

        def no_clips(thing):
            if not isinstance(thing, otio.schema.Clip):
                return thing
            return None

        result = otio.algorithms.filtered_composition(tr, no_clips)
        self.assertEqual(0, len(result))
        self.assertEqual(tr.metadata, result.metadata)

        # emptying the track should have the same effect
        del tr[:]
        self.assertIsOTIOEquivalentTo(tr, result)

    def test_prune_by_type_args(self):
        """Test pruning using the types_to_prune list"""

        md = {'test': 'bar'}
        tr = otio.schema.Track(name='foo', metadata=md)
        tr.append(otio.schema.Clip(name='cl1', metadata=md))

        result = otio.algorithms.filtered_composition(
            tr,
            lambda _: _,
            types_to_prune=(otio.schema.Clip,)
        )
        self.assertEqual(0, len(result))
        self.assertEqual(tr.metadata, result.metadata)

        # emptying the track should have the same effect
        del tr[:]
        self.assertIsOTIOEquivalentTo(tr, result)

    def test_copy(self):
        md = {'test': 'bar'}
        tl = otio.schema.Timeline(name='foo', metadata=md)
        tl.tracks.append(otio.schema.Track(name='track1', metadata=md))
        tl.tracks[0].append(otio.schema.Clip(name='cl1', metadata=md))

        test = otio.algorithms.filtered_composition(tl, lambda _: _)

        # make sure the original timeline didn't get nuked
        self.assertEqual(len(tl.tracks), 1)

        self.assertJsonEqual(tl, test)

    def test_insert_tuple(self):
        """test a reduce that takes each clip in a sequence and triples it"""

        md = {'test': 'bar'}
        tr = otio.schema.Track(name='foo', metadata=md)
        tr.append(otio.schema.Clip(name='cl1', metadata=md))

        def triple_clips(thing):
            if not isinstance(thing, otio.schema.Clip):
                return thing
            return (thing, copy.deepcopy(thing), copy.deepcopy(thing))

        result = otio.algorithms.filtered_composition(tr, triple_clips)
        self.assertEqual(3, len(result))
        self.assertEqual(tr.metadata, result.metadata)

        # emptying the track should have the same effect
        tr.extend([copy.deepcopy(tr[0]), copy.deepcopy(tr[0])])
        self.assertJsonEqual(tr, result)


class ReduceTest(unittest.TestCase, otio_test_utils.OTIOAssertions):
    maxDiff = None

    def test_copy_track(self):
        md = {'test': 'bar'}
        tr = otio.schema.Track(name='foo', metadata=md)
        tr.append(otio.schema.Clip(name='cl1', metadata=md))

        self.assertJsonEqual(
            tr,
            otio.algorithms.filtered_with_sequence_context(
                tr,
                # no op - ignore all arguments and return original thing
                lambda _, thing, __: thing
            )
        )

    def test_copy_stack(self):
        """Test a no op reduce that copies the timeline."""

        md = {'test': 'bar'}
        tr = otio.schema.Stack(name='foo', metadata=md)
        tr.append(otio.schema.Clip(name='cl1', metadata=md))

        result = otio.algorithms.filtered_with_sequence_context(
            tr,
            # no op - ignore all arguments and return original thing
            lambda _, thing, __: thing
        )
        self.assertJsonEqual(tr, result)
        self.assertIsNot(tr[0], result)

    def test_prune_clips(self):
        """test a reduce that removes clips"""

        md = {'test': 'bar'}
        tr = otio.schema.Track(name='foo', metadata=md)
        tr.append(otio.schema.Clip(name='cl1', metadata=md))

        def no_clips(_, thing, __):
            if not isinstance(thing, otio.schema.Clip):
                return thing
            return None

        result = otio.algorithms.filtered_with_sequence_context(tr, no_clips)
        self.assertEqual(0, len(result))
        self.assertEqual(tr.metadata, result.metadata)

        # emptying the track should have the same effect
        del tr[:]
        self.assertIsOTIOEquivalentTo(tr, result)

    def test_prune_clips_using_types_to_prune(self):
        """Test pruning otio.schema.clip using the types_to_prune argument"""

        md = {'test': 'bar'}
        tr = otio.schema.Track(name='foo', metadata=md)
        tr.append(otio.schema.Clip(name='cl1', metadata=md))

        result = otio.algorithms.filtered_with_sequence_context(
            tr,
            # no op - ignore all arguments and return original thing
            lambda _, thing, __: thing,
            # instead use types_to_prune
            types_to_prune=(otio.schema.Clip,)
        )
        self.assertEqual(0, len(result))
        self.assertEqual(tr.metadata, result.metadata)

        # emptying the track should have the same effect
        del tr[:]
        self.assertIsOTIOEquivalentTo(tr, result)

    def test_insert_tuple(self):
        """test a reduce that takes each clip in a sequence and triples it"""

        md = {'test': 'bar'}
        tr = otio.schema.Track(name='foo', metadata=md)
        tr.append(otio.schema.Clip(name='cl1', metadata=md))

        def triple_clips(_, thing, __):
            if not isinstance(thing, otio.schema.Clip):
                return thing
            return (thing, copy.deepcopy(thing), copy.deepcopy(thing))

        result = otio.algorithms.filtered_with_sequence_context(
            tr,
            triple_clips
        )
        self.assertEqual(3, len(result))
        self.assertEqual(tr.metadata, result.metadata)

        # emptying the track should have the same effect
        tr.extend((copy.deepcopy(tr[0]), copy.deepcopy(tr[0])))
        self.assertJsonEqual(tr, result)

    def test_prune_clips_after_transitions(self):
        """test a reduce that removes clips that follow transitions"""

        md = {'test': 'bar'}
        tr = otio.schema.Track(name='foo', metadata=md)
        for i in range(5):
            ind = str(i)
            if i in (2, 3):
                tr.append(otio.schema.Transition(name='should_be_pruned' + ind))
            tr.append(otio.schema.Clip(name='cl' + ind, metadata=md))

        def no_clips_after_transitions(prev, thing, __):
            if (
                isinstance(prev, otio.schema.Transition) or
                isinstance(thing, otio.schema.Transition)
            ):
                return None
            return thing

        result = otio.algorithms.filtered_with_sequence_context(
            tr,
            no_clips_after_transitions
        )

        # emptying the track of transitions and the clips they follow and
        # should have the same effect
        del tr[2:6]
        self.assertJsonEqual(tr, result)

        self.assertEqual(3, len(result))
        self.assertEqual(tr.metadata, result.metadata)

        # ...but that things have been properly deep copied
        self.assertIsNot(tr.metadata, result.metadata)

    def test_copy(self):
        """Test that a simple reduce results in a copy"""
        md = {'test': 'bar'}
        tl = otio.schema.Timeline(name='foo', metadata=md)
        tl.tracks.append(otio.schema.Track(name='track1', metadata=md))
        tl.tracks[0].append(otio.schema.Clip(name='cl1', metadata=md))

        test = otio.algorithms.filtered_with_sequence_context(
            tl,
            # no op - ignore all arguments and return original thing
            lambda _, thing, __: thing
        )

        # make sure the original timeline didn't get nuked
        self.assertEqual(len(tl.tracks), 1)
        self.assertJsonEqual(tl, test)

    def test_prune_correct_duplicate(self):
        """test a reduce that removes the correct duplicate clip"""

        md = {'test': 'bar'}
        tr = otio.schema.Track()
        tr.append(otio.schema.Clip(metadata=md))
        tr.append(otio.schema.Gap())
        tr.append(otio.schema.Clip(metadata=md))
        tr.append(otio.schema.Gap())
        tr.append(otio.schema.Clip(metadata=md))

        clips = []

        def no_clip_2(_, thing, __):
            if isinstance(thing, otio.schema.Clip):
                clips.append(thing)
                if len(clips) == 2:
                    return None
            return thing

        result = otio.algorithms.filtered_with_sequence_context(tr, no_clip_2)
        self.assertEqual(4, len(result))
        self.assertTrue(isinstance(result[0], otio.schema.Clip))
        self.assertTrue(isinstance(result[1], otio.schema.Gap))
        self.assertTrue(isinstance(result[2], otio.schema.Gap))
        self.assertTrue(isinstance(result[3], otio.schema.Clip))


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

import unittest
import sys

import opentimelineio as otio
import opentimelineio.test_utils as otio_test_utils


class V2dTests(unittest.TestCase, otio_test_utils.OTIOAssertions):
    def test_cons(self):
        v = otio.schema.V2d(1.0, 2.0)

        self.assertEqual(v.x, 1.0)
        self.assertEqual(v.y, 2.0)

        self.assertEqual(v[0], 1.0)
        self.assertEqual(v[1], 2.0)

    def test_str(self):
        v = otio.schema.V2d(1.0, 2.0)

        self.assertMultiLineEqual(
            str(v),
            'V2d(1.0, 2.0)'
        )

        self.assertMultiLineEqual(
            repr(v),
            'otio.schema.V2d(x=1.0, y=2.0)'
        )

    def test_equality(self):
        v1 = otio.schema.V2d(1.0, 2.0)
        v2 = otio.schema.V2d(3.0, 4.0)

        self.assertFalse(v1 == v2)
        self.assertTrue(v1 != v2)

        v3 = otio.schema.V2d(1.0, 2.0)
        self.assertTrue(v1 == v3)
        self.assertFalse(v1 != v3)

        self.assertTrue(v1.equalWithAbsError(v3, 0.0))
        self.assertTrue(v1.equalWithRelError(v3, 0.0))

    def test_math_ops(self):
        v1 = otio.schema.V2d(1.0, 2.0)
        v2 = otio.schema.V2d(3.0, 4.0)

        self.assertEqual(v1 ^ v2, 11.0)
        self.assertEqual(v1.dot(v2), 11.0)

        self.assertEqual(v1 % v2, -2.0)
        self.assertEqual(v1.cross(v2), -2.0)

        self.assertEqual(v1 + v2, otio.schema.V2d(4.0, 6.0))
        self.assertEqual(v1 - v2, otio.schema.V2d(-2.0, -2.0))
        self.assertEqual(v1 * v2, otio.schema.V2d(3.0, 8.0))
        self.assertEqual(v1 / v2, otio.schema.V2d(1.0 / 3.0, 0.5))

        v1 += v2
        self.assertEqual(v1, otio.schema.V2d(4.0, 6.0))

        v1 -= v2
        self.assertEqual(v1, otio.schema.V2d(1.0, 2.0))

        v1 *= v2
        self.assertEqual(v1, otio.schema.V2d(3.0, 8.0))

        v1 /= v2
        self.assertEqual(v1, otio.schema.V2d(1.0, 2.0))

    def test_geometry(self):
        v = otio.schema.V2d(3.0, 4.0)

        self.assertEqual(v.length(), 5.0)
        self.assertEqual(v.length2(), 25.0)

    def test_normalize(self):
        v = otio.schema.V2d(3.0, 4.0)

        self.assertEqual(v.normalized(), otio.schema.V2d(0.6, 0.8))
        self.assertEqual(v.normalizedNonNull(), otio.schema.V2d(0.6, 0.8))

        v2 = v
        v.normalize()
        v2.normalizeNonNull()
        self.assertEqual(v, otio.schema.V2d(0.6, 0.8))
        self.assertEqual(v2, otio.schema.V2d(0.6, 0.8))

        nv = otio.schema.V2d(0.0, 0.0)

        with self.assertRaises((ValueError, RuntimeError)):
            nv.normalizeExc()

        with self.assertRaises((ValueError, RuntimeError)):
            nv.normalizedExc()

    def test_limits(self):
        self.assertEqual(otio.schema.V2d.baseTypeLowest(), -1 * sys.float_info.max)
        self.assertEqual(otio.schema.V2d.baseTypeMax(), sys.float_info.max)
        self.assertEqual(otio.schema.V2d.baseTypeSmallest(), sys.float_info.min)
        self.assertEqual(otio.schema.V2d.baseTypeEpsilon(), sys.float_info.epsilon)


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

import unittest

import opentimelineio as otio
import opentimelineio.test_utils as otio_test_utils


class ClipTests(unittest.TestCase, otio_test_utils.OTIOAssertions):

    def test_cons(self):
        name = "test"
        rt = otio.opentime.RationalTime(5, 24)
        tr = otio.opentime.TimeRange(rt, rt)
        mr = otio.schema.ExternalReference(
            available_range=otio.opentime.TimeRange(
                rt,
                otio.opentime.RationalTime(10, 24)
            ),
            target_url="/var/tmp/test.mov"
        )

        cl = otio.schema.Clip(
            name=name,
            media_reference=mr,
            source_range=tr,
            # transition_in
            # transition_out
        )
        self.assertEqual(cl.name, name)
        self.assertEqual(cl.source_range, tr)
        self.assertIsOTIOEquivalentTo(cl.media_reference, mr)

        encoded = otio.adapters.otio_json.write_to_string(cl)
        decoded = otio.adapters.otio_json.read_from_string(encoded)
        self.assertIsOTIOEquivalentTo(cl, decoded)

    def test_each_clip(self):
        cl = otio.schema.Clip(name="test_clip")
        self.assertEqual(list(cl.each_clip()), [cl])

    def test_str(self):
        cl = otio.schema.Clip(name="test_clip")

        self.assertMultiLineEqual(
            str(cl),
            'Clip("test_clip", MissingReference(\'\', None, None, {}), None, {})'
        )
        self.assertMultiLineEqual(
            repr(cl),
            'otio.schema.Clip('
            "name='test_clip', "
            'media_reference={}, '
            'source_range=None, '
            'metadata={{}}'
            ')'.format(
                repr(cl.media_reference)
            )
        )

    def test_str_with_filepath(self):
        cl = otio.schema.Clip(
            name="test_clip",
            media_reference=otio.schema.ExternalReference(
                "/var/tmp/foo.mov"
            )
        )
        self.assertMultiLineEqual(
            str(cl),
            'Clip('
            '"test_clip", ExternalReference("/var/tmp/foo.mov"), None, {}'
            ')'
        )
        self.assertMultiLineEqual(
            repr(cl),
            'otio.schema.Clip('
            "name='test_clip', "
            "media_reference=otio.schema.ExternalReference("
            "target_url='/var/tmp/foo.mov'"
            "), "
            'source_range=None, '
            'metadata={}'
            ')'
        )

    def test_ranges(self):
        tr = otio.opentime.TimeRange(
            # 1 hour in at 24 fps
            start_time=otio.opentime.RationalTime(86400, 24),
            duration=otio.opentime.RationalTime(200, 24)
        )

        cl = otio.schema.Clip(
            name="test_clip",
            media_reference=otio.schema.ExternalReference(
                "/var/tmp/foo.mov",
                available_range=tr
            )
        )
        self.assertEqual(cl.duration(), cl.trimmed_range().duration)
        self.assertEqual(cl.duration(), tr.duration)
        self.assertEqual(cl.trimmed_range(), tr)
        self.assertEqual(cl.available_range(), tr)
        self.assertIsNot(cl.trimmed_range(), tr)
        self.assertIsNot(cl.available_range(), tr)

        cl.source_range = otio.opentime.TimeRange(
            # 1 hour + 100 frames
            start_time=otio.opentime.RationalTime(86500, 24),
            duration=otio.opentime.RationalTime(50, 24)
        )
        self.assertNotEqual(cl.duration(), tr.duration)
        self.assertNotEqual(cl.trimmed_range(), tr)
        self.assertEqual(cl.duration(), cl.source_range.duration)
        self.assertIsNot(cl.duration(), cl.source_range.duration)

        self.assertEqual(cl.trimmed_range(), cl.source_range)
        self.assertIsNot(cl.trimmed_range(), cl.source_range)

    def test_available_image_bounds(self):
        available_image_bounds = otio.schema.Box2d(
            otio.schema.V2d(0.0, 0.0),
            otio.schema.V2d(16.0, 9.0)
        )

        media_reference = otio.schema.ExternalReference(
            "/var/tmp/foo.mov",
            available_image_bounds=available_image_bounds
        )

        cl = otio.schema.Clip(
            name="test_available_image_bounds",
            media_reference=media_reference
        )

        self.assertEqual(available_image_bounds, cl.available_image_bounds)
        self.assertEqual(
            cl.available_image_bounds,
            media_reference.available_image_bounds
        )

        self.assertEqual(0.0, cl.available_image_bounds.min.x)
        self.assertEqual(0.0, cl.available_image_bounds.min.y)
        self.assertEqual(16.0, cl.available_image_bounds.max.x)
        self.assertEqual(9.0, cl.available_image_bounds.max.y)

        # test range exceptions
        cl.media_reference.available_image_bounds = None
        with self.assertRaises(otio.exceptions.CannotComputeAvailableRangeError):
            cl.available_range()

    def test_ref_default(self):
        cl = otio.schema.Clip()
        self.assertIsOTIOEquivalentTo(
            cl.media_reference,
            otio.schema.MissingReference()
        )

        cl.media_reference = None
        self.assertIsOTIOEquivalentTo(
            cl.media_reference,
            otio.schema.MissingReference()
        )

        cl.media_reference = otio.schema.ExternalReference()
        self.assertIsOTIOEquivalentTo(
            cl.media_reference,
            otio.schema.ExternalReference()
        )

    def test_multi_ref(self):
        cl = otio.schema.Clip()

        self.assertEqual(
            otio.schema.Clip.DEFAULT_MEDIA_KEY,
            cl.active_media_reference_key
        )
        self.assertIsOTIOEquivalentTo(
            cl.media_reference,
            otio.schema.MissingReference()
        )

        mrs = cl.media_references()
        self.assertIsOTIOEquivalentTo(
            mrs[otio.schema.Clip.DEFAULT_MEDIA_KEY],
            otio.schema.MissingReference()
        )

        cl.set_media_references(
            {
                otio.schema.Clip.DEFAULT_MEDIA_KEY: otio.schema.ExternalReference(),
                "high_quality": otio.schema.GeneratorReference(),
                "proxy_quality": otio.schema.ImageSequenceReference(),
            },
            otio.schema.Clip.DEFAULT_MEDIA_KEY)

        mrs = cl.media_references()
        self.assertIsOTIOEquivalentTo(
            mrs[otio.schema.Clip.DEFAULT_MEDIA_KEY],
            otio.schema.ExternalReference()
        )
        self.assertIsOTIOEquivalentTo(
            mrs["high_quality"],
            otio.schema.GeneratorReference()
        )
        self.assertIsOTIOEquivalentTo(
            mrs["proxy_quality"],
            otio.schema.ImageSequenceReference()
        )

        cl.active_media_reference_key = "high_quality"
        self.assertIsOTIOEquivalentTo(
            cl.media_reference,
            otio.schema.GeneratorReference()
        )
        self.assertEqual(
            cl.active_media_reference_key,
            "high_quality"
        )

        # we should get an exception if we try to use a key that is
        # not in the media_references
        with self.assertRaises(ValueError):
            cl.active_media_reference_key = "cloud"

        self.assertEqual(
            cl.active_media_reference_key,
            "high_quality"
        )

        # we should also get an exception if we set the references without
        # the active key
        with self.assertRaises(ValueError):
            cl.set_media_references(
                {
                    "cloud": otio.schema.ExternalReference()
                },
                "high_quality"
            )
        self.assertEqual(
            cl.active_media_reference_key,
            "high_quality"
        )

        # we should also get an exception if we set the references with
        # an empty key
        with self.assertRaises(ValueError):
            cl.set_media_references(
                {
                    "": otio.schema.ExternalReference()
                },
                ""
            )
        self.assertEqual(
            cl.active_media_reference_key,
            "high_quality"
        )

        # setting the references and the active key should resolve the problem
        cl.set_media_references(
            {
                "cloud": otio.schema.ExternalReference()
            },
            "cloud"
        )
        self.assertEqual(cl.active_media_reference_key, "cloud")
        self.assertIsOTIOEquivalentTo(
            cl.media_reference, otio.schema.ExternalReference())


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""Unsupported contrib code for OpenTimelineIO."""

# flake8: noqa

from . import (
    adapters
)



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

"""This file is here to support the test_adapter_plugin unittest.
If you want to learn how to write your own adapter plugin, please read:
https://opentimelineio.readthedocs.io/en/latest/tutorials/write-an-adapter.html
"""

import opentimelineio as otio


def read_from_file(filepath, suffix=""):
    fake_tl = otio.schema.Timeline(name=filepath + str(suffix))
    fake_tl.tracks.append(otio.schema.Track())
    fake_tl.tracks[0].append(otio.schema.Clip(name=filepath + "_clip"))
    return fake_tl


def read_from_string(input_str, suffix=""):
    return read_from_file(input_str, suffix)


# in practice, these will be in separate plugins, but for simplicity in the
# unit tests, we put this in the same file as the example adapter.
def link_media_reference(in_clip, media_linker_argument_map):
    d = {'from_test_linker': True}
    d.update(media_linker_argument_map)
    return otio.schema.MissingReference(
        name=in_clip.name + "_tweaked",
        metadata=d
    )


# same thing for this hookscript
def hook_function(in_timeline, argument_map=None):
    in_timeline.name = "hook ran and did stuff"
    in_timeline.metadata.update(argument_map)
    return in_timeline



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

import unittest
import opentimelineio as otio
import opentimelineio.test_utils as otio_test_utils
import opentimelineio_contrib.adapters.kdenlive as kdenlive_adapter
import os
from xml.etree import ElementTree as ET


def prepare_for_check(timeline):
    """Clear the given timeline of irrelevant data
    For example since Kdenlive only supports one timeline,
    we do not care about its name. Same applies to reference names."""
    timeline.name = ""
    for track in timeline.tracks:
        for clip in track.clip_if():
            if isinstance(clip.media_reference, list):
                for reference in clip.media_reference:
                    reference.name = ""
            else:
                clip.media_reference.name = ""


class AdaptersKdenliveTest(unittest.TestCase, otio_test_utils.OTIOAssertions):

    def __init__(self, *args, **kwargs):
        super(AdaptersKdenliveTest, self).__init__(*args, **kwargs)

    def test_library_roundtrip(self):
        timeline = otio.adapters.read_from_file(
            os.path.join(os.path.dirname(__file__), "sample_data",
                         "kdenlive_example_v221170.kdenlive"))

        # check tracks
        self.assertIsNotNone(timeline)
        self.assertEqual(len(timeline.tracks), 5)

        self.assertEqual(len(timeline.video_tracks()), 2)
        self.assertEqual(len(timeline.audio_tracks()), 3)

        # check clips
        clip_urls = (('AUD0002.OGG',),
                     ('AUD0001.OGG', 'AUD0001.OGG'),
                     ('VID0001.MKV', 'VID0001.MKV'),
                     ('VID0001.MKV', 'VID0001.MKV'),
                     ('VID0002.MKV', 'VID0003.MKV'))

        for n, track in enumerate(timeline.tracks):
            self.assertTupleEqual(
                clip_urls[n],
                tuple(c.media_reference.target_url
                      for c in track
                      if isinstance(c, otio.schema.Clip) and
                      isinstance(
                          c.media_reference,
                          otio.schema.ExternalReference)))

        # check timeline markers
        self.assertEqual(len(timeline.tracks.markers), 2)

        markers_data = ((230, 'Purple Marker', otio.schema.MarkerColor.PURPLE),
                        (466, 'Green', otio.schema.MarkerColor.GREEN))

        for n, marker in enumerate(timeline.tracks.markers):
            self.assertEqual(0, marker.marked_range.duration.value)
            self.assertEqual(markers_data[n][0],
                             marker.marked_range.start_time.value)
            self.assertEqual(markers_data[n][1], marker.name)
            self.assertEqual(markers_data[n][2], marker.color)

        kdenlive_xml = otio.adapters.write_to_string(timeline, "kdenlive")
        self.assertIsNotNone(kdenlive_xml)

        new_timeline = otio.adapters.read_from_string(kdenlive_xml, "kdenlive")
        self.assertJsonEqual(timeline, new_timeline)

    def test_v19_11_80__file_roundtrip(self):
        timeline = otio.adapters.read_from_file(
            os.path.join(os.path.dirname(__file__), "sample_data",
                         "kdenlive_example_v191180.kdenlive"))

        self.assertIsNotNone(timeline)
        self.assertEqual(len(timeline.tracks), 5)

        self.assertEqual(len(timeline.video_tracks()), 2)
        self.assertEqual(len(timeline.audio_tracks()), 3)

        clip_urls = (('AUD0002.OGG',),
                     ('AUD0001.OGG', 'AUD0001.OGG'),
                     ('VID0001.MKV', 'VID0001.MKV'),
                     ('VID0001.MKV', 'VID0001.MKV'),
                     ('VID0002.MKV', 'VID0003.MKV'))

        for n, track in enumerate(timeline.tracks):
            self.assertTupleEqual(
                clip_urls[n],
                tuple(c.media_reference.target_url
                      for c in track
                      if isinstance(c, otio.schema.Clip) and
                      isinstance(
                          c.media_reference,
                          otio.schema.ExternalReference)))

        kdenlive_xml = otio.adapters.write_to_string(timeline, "kdenlive")
        self.assertIsNotNone(kdenlive_xml)

        new_timeline = otio.adapters.read_from_string(kdenlive_xml, "kdenlive")
        self.assertJsonEqual(timeline, new_timeline)

    def test_from_fcp_example(self):
        timeline = otio.adapters.read_from_file(
            os.path.join(
                os.path.dirname(__file__),
                "sample_data",
                "kdenlive_example_from_fcp.xml",
            ),
        )

        kdenlive_xml = otio.adapters.write_to_string(timeline, "kdenlive")
        self.assertIsNotNone(kdenlive_xml)

        new_timeline = otio.adapters.read_from_string(kdenlive_xml, "kdenlive")
        troublesome_clip = new_timeline.video_tracks()[0][35]
        self.assertEqual(
            troublesome_clip.source_range.duration.value,
            807,
        )

    def test_read_mixes(self):
        # mixes are yet only supported to be read, not written
        timeline = otio.adapters.read_from_file(
            os.path.join(os.path.dirname(__file__), "sample_data",
                         "kdenlive_mixes_markers.kdenlive"))

        # check tracks
        self.assertIsNotNone(timeline)
        self.assertEqual(len(timeline.tracks), 4)

        video_tracks = timeline.video_tracks()
        audio_tracks = timeline.audio_tracks()
        self.assertEqual(len(video_tracks), 2)
        self.assertEqual(len(audio_tracks), 2)

        # check items
        video_track_normal = video_tracks[0]
        video_track_mix = video_tracks[1]

        audio_track_normal = audio_tracks[1]
        audio_track_mix = audio_tracks[0]

        self.assertEqual(len(video_track_normal), 10)
        self.assertEqual(len(audio_track_normal), 10)
        self.assertEqual(len(video_track_mix), 15)
        self.assertEqual(len(audio_track_mix), 15)

        clips_normal = list(video_track_normal.clip_if())
        self.assertEqual(len(clips_normal), 8)
        clips_mix = list(video_track_mix.clip_if())
        self.assertEqual(len(clips_mix), 8)

        normal_item_order = [
            otio.schema.Clip,
            otio.schema.Clip,
            otio.schema.Gap,
            otio.schema.Clip,
            otio.schema.Clip,
            otio.schema.Clip,
            otio.schema.Clip,
            otio.schema.Gap,
            otio.schema.Clip,
            otio.schema.Clip
        ]
        self.assertEqual(
            [type(item) for item in video_track_normal],
            normal_item_order
        )
        self.assertEqual(
            [type(item) for item in audio_track_normal],
            normal_item_order
        )

        mix_item_order = [
            otio.schema.Clip,
            otio.schema.Transition,
            otio.schema.Clip,
            otio.schema.Gap,
            otio.schema.Clip,
            otio.schema.Transition,
            otio.schema.Clip,
            otio.schema.Transition,
            otio.schema.Clip,
            otio.schema.Transition,
            otio.schema.Clip,
            otio.schema.Gap,
            otio.schema.Clip,
            otio.schema.Transition,
            otio.schema.Clip
        ]
        self.assertEqual(
            [type(item) for item in video_track_mix],
            mix_item_order
        )
        self.assertEqual(
            [type(item) for item in audio_track_mix],
            mix_item_order
        )

        def only_transitions(item):
            return isinstance(item, otio.schema.Transition)

        mix_times = ((13, 25),
                     (25, 25),
                     (0, 25),
                     (115, 214),
                     (44, 114))
        for x, mix in enumerate(filter(only_transitions, video_track_mix)):
            duration = mix.in_offset.value + mix.out_offset.value
            self.assertEqual(mix.in_offset.value, mix_times[x][0])
            self.assertEqual(duration, mix_times[x][1])

        mix_times = ((13, 25),
                     (13, 25),
                     (119, 131),
                     (13, 25),
                     (44, 114))
        for x, mix in enumerate(filter(only_transitions, audio_track_mix)):
            duration = mix.in_offset.value + mix.out_offset.value
            self.assertEqual(mix.in_offset.value, mix_times[x][0])
            self.assertEqual(duration, mix_times[x][1])

    def test_fun_read_mix(self):
        rate = 25
        mix = ET.Element(
            'transition',
            {"in": '00:00:11.000', "out": '00:00:13.000'},
        )
        mixcut = ET.SubElement(mix, 'property', {'name': 'kdenlive:mixcut'})
        mixcut.text = '16'
        reverese_prop = ET.SubElement(mix, 'property', {'name': 'reverse'})
        reverese_prop.text = '1'
        (mix_range, before_mix_cut,
         after_mix_cut, reverse) = kdenlive_adapter.read_mix(mix, rate)
        self.assertIsNotNone(mix_range)
        self.assertIsNotNone(before_mix_cut)
        self.assertIsNotNone(after_mix_cut)
        self.assertIsNotNone(reverse)
        self.assertEqual(mix_range, otio.opentime.TimeRange(
            start_time=otio.opentime.RationalTime(11 * rate, rate),
            duration=otio.opentime.RationalTime(2 * rate, rate)
        ))
        self.assertEqual(before_mix_cut, otio.opentime.RationalTime(16, rate))
        self.assertEqual(after_mix_cut,
                         otio.opentime.RationalTime(2 * rate, rate)
                         - otio.opentime.RationalTime(16, rate))
        self.assertEqual(reverse, True)

    def test_read_clip_markers(self):
        # clip markers are yet only supported to be read, not written
        timeline = otio.adapters.read_from_file(
            os.path.join(os.path.dirname(__file__), "sample_data",
                         "kdenlive_mixes_markers.kdenlive"))

        # check tracks
        self.assertIsNotNone(timeline)
        self.assertEqual(len(timeline.tracks), 4)

        video_tracks = timeline.video_tracks()
        audio_tracks = timeline.audio_tracks()
        self.assertEqual(len(video_tracks), 2)
        self.assertEqual(len(audio_tracks), 2)

        def only_clips(item):
            return isinstance(item, otio.schema.Clip)

        for track in timeline.tracks:
            for clip in filter(only_clips, track):
                self.assertEqual(len(clip.markers), 2)

                markers_data = (
                    (1782, 'Lila', otio.schema.MarkerColor.PURPLE),
                    (2899, 'Orange', otio.schema.MarkerColor.ORANGE))

                for n, marker in enumerate(clip.markers):
                    self.assertEqual(0, marker.marked_range.duration.value)
                    self.assertEqual(markers_data[n][0],
                                     marker.marked_range.start_time.value)
                    self.assertEqual(markers_data[n][1], marker.name)
                    self.assertEqual(markers_data[n][2], marker.color)

    def test_smpte_bars(self):
        timeline = otio.adapters.read_from_file(
            os.path.join(
                os.path.dirname(__file__),
                "sample_data",
                "generator_reference_test.otio",
            ),
        )

        kdenlive_xml = otio.adapters.write_to_string(timeline, "kdenlive")
        self.assertIsNotNone(kdenlive_xml)

        new_timeline = otio.adapters.read_from_string(kdenlive_xml, "kdenlive")
        prepare_for_check(timeline)
        prepare_for_check(new_timeline)
        self.assertIsOTIOEquivalentTo(timeline, new_timeline)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

import unittest
import os
import pkg_resources
import sys

try:
    # Python 3.3 forward includes the mock module
    from unittest import mock
    could_import_mock = True
except ImportError:
    # Fallback for older python (not included in standard library)
    try:
        import mock
        could_import_mock = True
    except ImportError:
        # Mock appears to not be installed
        could_import_mock = False

try:
    # Python3:  use importlib.reload
    from importlib import reload as import_reload
except ImportError:
    # Python2:
    from imp import reload as import_reload

import opentimelineio as otio
from tests import baseline_reader


@unittest.skipIf(
    not could_import_mock,
    "mock module not found. Install mock from pypi or use python >= 3.3."
)
class TestSetuptoolsPlugin(unittest.TestCase):
    def setUp(self):
        # Get the location of the mock plugin module metadata
        mock_module_path = os.path.join(
            os.path.normpath(baseline_reader.path_to_baseline_directory()),
            'plugin_module',
        )
        self.mock_module_manifest_path = os.path.join(
            mock_module_path,
            "otio_jsonplugin",
            "plugin_manifest.json"
        )

        self.override_adapter_manifest_path = os.path.join(
            mock_module_path,
            "otio_override_adapter",
            "plugin_manifest.json"
        )

        # Create a WorkingSet as if the module were installed
        entries = [mock_module_path] + pkg_resources.working_set.entries

        self.sys_patch = mock.patch('sys.path', entries)
        self.sys_patch.start()

        working_set = pkg_resources.WorkingSet(entries)

        # linker from the entry point
        self.entry_patcher = mock.patch(
            'pkg_resources.iter_entry_points',
            working_set.iter_entry_points
        )
        self.entry_patcher.start()

    def tearDown(self):
        self.sys_patch.stop()
        self.entry_patcher.stop()
        if 'otio_mockplugin' in sys.modules:
            del sys.modules['otio_mockplugin']

        if 'otio_override_adapter' in sys.modules:
            del sys.modules['otio_override_adapter']

    def test_detect_plugin(self):
        """This manifest uses the plugin_manifest function"""

        # Create a manifest and ensure it detected the mock adapter and linker
        man = otio.plugins.manifest.load_manifest()

        # Make sure the adapter is included in the adapter list
        adapter_names = [adapter.name for adapter in man.adapters]
        self.assertIn('mock_adapter', adapter_names)

        # Make sure the linker is included in the linker list
        linker_names = [linker.name for linker in man.media_linkers]
        self.assertIn('mock_linker', linker_names)

        # Make sure adapters and linkers landed in the proper place
        for adapter in man.adapters:
            self.assertIsInstance(adapter, otio.adapters.Adapter)

        for linker in man.media_linkers:
            self.assertIsInstance(linker, otio.media_linker.MediaLinker)

    def test_overrride_adapter(self):
        # Test that entrypoint plugins load before builtin and contrib
        man = otio.plugins.manifest.load_manifest()

        # The override_adapter creates another cmx_3600 adapter
        adapters = [adapter for adapter in man.adapters
                    if adapter.name == "cmx_3600"]

        # More then one cmx_3600 adapter should exist.
        self.assertTrue(len(adapters) > 1)

        # Override adapter should be the first adapter found
        manifest = adapters[0].plugin_info_map().get('from manifest', None)
        self.assertEqual(manifest, os.path.abspath(self.override_adapter_manifest_path))

        self.assertTrue(
            any(
                True for p in man.source_files
                if self.override_adapter_manifest_path in p
            )
        )

    def test_pkg_resources_disabled(self):
        os.environ["OTIO_DISABLE_PKG_RESOURCE_PLUGINS"] = "1"
        import_reload(otio.plugins.manifest)

        # detection of the environment variable happens on import, force a
        # reload to ensure that it is triggered
        with self.assertRaises(AssertionError):
            self.test_detect_plugin()

        # override adapter should not be loaded either
        with self.assertRaises(AssertionError):
            self.test_overrride_adapter()

        # remove the environment variable and reload again for usage in the
        # other tests
        del os.environ["OTIO_DISABLE_PKG_RESOURCE_PLUGINS"]
        import_reload(otio.plugins.manifest)

    def test_detect_plugin_json_manifest(self):
        # Test detecting a plugin that rather than exposing the plugin_manifest
        # function, just simply has a plugin_manifest.json provided at the
        # package top level.
        man = otio.plugins.manifest.load_manifest()

        # Make sure the adapter is included in the adapter list
        adapter_names = [adapter.name for adapter in man.adapters]
        self.assertIn('mock_adapter_json', adapter_names)

        # Make sure the linker is included in the linker list
        linker_names = [linker.name for linker in man.media_linkers]
        self.assertIn('mock_linker_json', linker_names)

        # Make sure adapters and linkers landed in the proper place
        for adapter in man.adapters:
            self.assertIsInstance(adapter, otio.adapters.Adapter)

        for linker in man.media_linkers:
            self.assertIsInstance(linker, otio.media_linker.MediaLinker)

        self.assertTrue(
            any(
                True for p in man.source_files
                if self.mock_module_manifest_path in p
            )
        )

    def test_deduplicate_env_variable_paths(self):
        "Ensure that duplicate entries in the environment variable are ignored"

        # back up existing manifest
        bak_env = os.environ.get('OTIO_PLUGIN_MANIFEST_PATH')

        relative_path = self.mock_module_manifest_path.replace(os.getcwd(), '.')

        # set where to find the new manifest
        os.environ['OTIO_PLUGIN_MANIFEST_PATH'] = os.pathsep.join(
            (
                # absolute
                self.mock_module_manifest_path,

                # relative
                relative_path
            )
        )

        result = otio.plugins.manifest.load_manifest()
        self.assertEqual(
            len(
                [
                    p for p in result.source_files
                    if self.mock_module_manifest_path in p
                ]
            ),
            1
        )
        if relative_path != self.mock_module_manifest_path:
            self.assertNotIn(relative_path, result.source_files)

        if bak_env:
            os.environ['OTIO_PLUGIN_MANIFEST_PATH'] = bak_env
        else:
            del os.environ['OTIO_PLUGIN_MANIFEST_PATH']


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

import os
import subprocess
import sys
import unittest
import opentimelineio as otio
import opentimelineio.test_utils as otio_test_utils
from opentimelineio_contrib.adapters.fcpx_xml import format_name

try:
    # Python 3.3 forward includes the mock module
    from unittest import mock
    could_import_mock = True
except ImportError:
    # Fallback for older python (not included in standard library)
    try:
        import mock
        could_import_mock = True
    except ImportError:
        # Mock appears to not be installed
        could_import_mock = False

SAMPLE_LIBRARY_XML = os.path.join(
    os.path.dirname(__file__),
    "sample_data",
    "fcpx_library.fcpxml"
)
SAMPLE_PROJECT_XML = os.path.join(
    os.path.dirname(__file__),
    "sample_data",
    "fcpx_project.fcpxml"
)
SAMPLE_EVENT_XML = os.path.join(
    os.path.dirname(__file__),
    "sample_data",
    "fcpx_event.fcpxml"
)
SAMPLE_CLIPS_XML = os.path.join(
    os.path.dirname(__file__),
    "sample_data",
    "fcpx_clips.fcpxml"
)


class AdaptersFcpXXmlTest(unittest.TestCase, otio_test_utils.OTIOAssertions):
    """
    The test class for the FCP X XML adapter
    """

    def __init__(self, *args, **kwargs):
        super(AdaptersFcpXXmlTest, self).__init__(*args, **kwargs)
        self.maxDiff = None

    def test_library_roundtrip(self):
        container = otio.adapters.read_from_file(SAMPLE_LIBRARY_XML)
        timeline = next(
            container.each_child(descended_from_type=otio.schema.Timeline)
        )

        self.assertIsNotNone(timeline)
        self.assertEqual(len(timeline.tracks), 4)

        self.assertEqual(len(timeline.video_tracks()), 3)
        self.assertEqual(len(timeline.audio_tracks()), 1)

        video_clip_names = (
            (
                'IMG_0715',
                "",
                'compound_clip_1',
                'IMG_0233',
                'IMG_0687',
                'IMG_0268',
                'compound_clip_1'
            ),
            ("", 'IMG_0513', "", 'IMG_0268', 'IMG_0740'),
            ("", 'IMG_0857')
        )

        for n, track in enumerate(timeline.video_tracks()):
            self.assertTupleEqual(
                tuple(c.name for c in track),
                video_clip_names[n]
            )

        fcpx_xml = otio.adapters.write_to_string(container, "fcpx_xml")
        self.assertIsNotNone(fcpx_xml)

        new_timeline = otio.adapters.read_from_string(fcpx_xml, "fcpx_xml")
        self.assertJsonEqual(container, new_timeline)

    def test_event_roundtrip(self):
        container = otio.adapters.read_from_file(SAMPLE_EVENT_XML)
        timeline = next(
            container.each_child(descended_from_type=otio.schema.Timeline)
        )

        self.assertIsNotNone(timeline)
        self.assertEqual(len(timeline.tracks), 4)

        self.assertEqual(len(timeline.video_tracks()), 3)
        self.assertEqual(len(timeline.audio_tracks()), 1)

        video_clip_names = (
            (
                'IMG_0715',
                "",
                'compound_clip_1',
                'IMG_0233',
                'IMG_0687',
                'IMG_0268',
                'compound_clip_1'
            ),
            ("", 'IMG_0513', "", 'IMG_0268', 'IMG_0740'),
            ("", 'IMG_0857')
        )

        for n, track in enumerate(timeline.video_tracks()):
            self.assertTupleEqual(
                tuple(c.name for c in track),
                video_clip_names[n]
            )

        fcpx_xml = otio.adapters.write_to_string(container, "fcpx_xml")
        self.assertIsNotNone(fcpx_xml)

        new_timeline = otio.adapters.read_from_string(fcpx_xml, "fcpx_xml")
        self.assertJsonEqual(container, new_timeline)

    def test_project_roundtrip(self):
        timeline = otio.adapters.read_from_file(SAMPLE_PROJECT_XML)

        self.assertIsNotNone(timeline)
        self.assertEqual(len(timeline.tracks), 4)

        self.assertEqual(len(timeline.video_tracks()), 3)
        self.assertEqual(len(timeline.audio_tracks()), 1)

        video_clip_names = (
            (
                'IMG_0715',
                "",
                'compound_clip_1',
                'IMG_0233',
                'IMG_0687',
                'IMG_0268',
                'compound_clip_1'
            ),
            ("", 'IMG_0513', "", 'IMG_0268', 'IMG_0740'),
            ("", 'IMG_0857')
        )

        for n, track in enumerate(timeline.video_tracks()):
            self.assertTupleEqual(
                tuple(c.name for c in track),
                video_clip_names[n]
            )

        fcpx_xml = otio.adapters.write_to_string(timeline, "fcpx_xml")
        self.assertIsNotNone(fcpx_xml)

        new_timeline = otio.adapters.read_from_string(fcpx_xml, "fcpx_xml")
        self.assertJsonEqual(timeline, new_timeline)

    def test_clips_roundtrip(self):
        container = otio.adapters.read_from_file(SAMPLE_CLIPS_XML)
        fcpx_xml = otio.adapters.write_to_string(container, "fcpx_xml")
        self.assertIsNotNone(fcpx_xml)

        new_timeline = otio.adapters.read_from_string(fcpx_xml, "fcpx_xml")
        self.assertJsonEqual(container, new_timeline)

    @unittest.skipIf(
        not could_import_mock,
        "mock module not found. Install mock from pypi or use python >= 3.3."
    )
    def test_format_name(self):
        rvalue = subprocess.check_output([sys.executable, '-c', 'print("640x360")'])
        with mock.patch.object(subprocess, 'check_output', return_value=rvalue):
            with mock.patch.object(os.path, 'exists', return_value=True):
                self.assertEqual(format_name(
                    25, "file:///dummy.me"), 'FFVideoFormat640x360p25')


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
# Copyright Contributors to the OpenTimelineIO project

import unittest
import os
from copy import deepcopy

import opentimelineio as otio

import opentimelineio.test_utils as otio_test_utils

from tests import (
    baseline_reader,
    utils,
)

# handle python2 vs python3 difference
try:
    from tempfile import TemporaryDirectory  # noqa: F401
    import tempfile
except ImportError:
    # XXX: python2.7 only
    from backports import tempfile


HOOKSCRIPT_PATH = "hookscript_example"
POST_WRITE_HOOKSCRIPT_PATH = "post_write_hookscript_example"

POST_RUN_NAME = "hook ran and did stuff"
TEST_METADATA = {'extra_data': True}


class HookScriptTest(unittest.TestCase, otio_test_utils.OTIOAssertions):
    """Tests for the hook function plugins."""
    def setUp(self):

        self.jsn = baseline_reader.json_baseline_as_string(HOOKSCRIPT_PATH)
        self.hook_script = otio.adapters.read_from_string(
            self.jsn,
            'otio_json'
        )
        self.hook_script._json_path = os.path.join(
            baseline_reader.MODPATH,
            "baselines",
            HOOKSCRIPT_PATH
        )

    def test_plugin_hook(self):
        self.assertEqual(self.hook_script.name, "example hook")
        self.assertEqual(self.hook_script.execution_scope, "in process")
        self.assertEqual(self.hook_script.filepath, "example.py")

    def test_plugin_hook_runs(self):
        tl = otio.schema.Timeline()
        tl = self.hook_script.run(tl)
        self.assertEqual(tl.name, POST_RUN_NAME)


class TestPluginHookSystem(unittest.TestCase):
    """ Test the hook point definition system """
    def setUp(self):
        self.man = utils.create_manifest()
        self.jsn = baseline_reader.json_baseline_as_string(HOOKSCRIPT_PATH)
        self.hsf = otio.adapters.otio_json.read_from_string(self.jsn)
        self.hsf._json_path = os.path.join(
            baseline_reader.MODPATH,
            "baselines",
            HOOKSCRIPT_PATH
        )
        self.post_jsn = baseline_reader.json_baseline_as_string(
            POST_WRITE_HOOKSCRIPT_PATH
        )
        self.post_hsf = otio.adapters.otio_json.read_from_string(
            self.post_jsn
        )
        self.post_hsf._json_path = os.path.join(
            baseline_reader.MODPATH,
            "baselines",
            POST_WRITE_HOOKSCRIPT_PATH
        )
        self.man.hook_scripts = [self.hsf, self.post_hsf]

        self.orig_manifest = otio.plugins.manifest._MANIFEST
        otio.plugins.manifest._MANIFEST = self.man

    def tearDown(self):
        utils.remove_manifest(self.man)
        otio.plugins.manifest._MANIFEST = self.orig_manifest

    def test_plugin_adapter(self):
        self.assertEqual(self.hsf.name, "example hook")
        self.assertEqual(self.hsf.execution_scope, "in process")
        self.assertEqual(self.hsf.filepath, "example.py")

    def test_load_adapter_module(self):
        target = os.path.join(
            baseline_reader.MODPATH,
            "baselines",
            "example.py"
        )

        self.assertEqual(self.hsf.module_abs_path(), target)
        self.assertTrue(hasattr(self.hsf.module(), "hook_function"))

    def test_run_hook_function(self):
        tl = otio.schema.Timeline()

        result = self.hsf.run(tl, TEST_METADATA)
        self.assertEqual(result.name, POST_RUN_NAME)
        self.assertEqual(result.metadata.get("extra_data"), True)

    def test_run_hook_through_adapters(self):
        result = otio.adapters.read_from_string(
            'foo', adapter_name='example',
            media_linker_name='example',
            hook_function_argument_map=TEST_METADATA
        )

        self.assertEqual(result.name, POST_RUN_NAME)
        self.assertEqual(result.metadata.get("extra_data"), True)

    def test_post_write_hook(self):
        self.man.adapters.extend(self.orig_manifest.adapters)

        tl = otio.schema.Timeline()
        tl_copy = deepcopy(tl)

        with tempfile.TemporaryDirectory() as temp_dir:
            filename = os.path.join(temp_dir, "post_hook_unittest.otio")
            arg_map = dict()
            otio.adapters.write_to_file(
                tl,
                filename,
                adapter_name='otio_json',
                hook_function_argument_map=arg_map
            )

            self.assertTrue(os.path.exists(filename))
            self.assertEqual(
                os.path.getsize(filename),
                tl.metadata.get('filesize')
            )

            # In this case the post hook actually changed the metadata.
            # Users must take care to catch changes they do in their hooks.
            self.assertNotEqual(tl, tl_copy)

    def test_serialize(self):

        self.assertEqual(
            str(self.hsf),
            "HookScript({}, {}, {})".format(
                repr(self.hsf.name),
                repr(self.hsf.execution_scope),
                repr(self.hsf.filepath)
            )
        )
        self.assertEqual(
            repr(self.hsf),
            "otio.hooks.HookScript("
            "name={}, "
            "execution_scope={}, "
            "filepath={}"
            ")".format(
                repr(self.hsf.name),
                repr(self.hsf.execution_scope),
                repr(self.hsf.filepath)
            )
        )

    def test_available_hookscript_names(self):
        # for not just assert that it returns a non-empty list
        self.assertEqual(
            list(otio.hooks.available_hookscripts()),
            [self.hsf, self.post_hsf]
        )
        self.assertEqual(
            otio.hooks.available_hookscript_names(),
            [self.hsf.name, self.post_hsf.name]
        )

    def test_manifest_hooks(self):
        self.assertEqual(
            sorted(list(otio.hooks.names())),
            sorted(
                ["post_adapter_read", "post_media_linker",
                 "pre_adapter_write", "post_adapter_write"]
            )
        )

        self.assertEqual(
            list(otio.hooks.scripts_attached_to("pre_adapter_write")),
            [
                self.hsf.name,
                self.hsf.name
            ]
        )

        self.assertEqual(
            list(otio.hooks.scripts_attached_to("post_adapter_read")),
            []
        )

        self.assertEqual(
            list(otio.hooks.scripts_attached_to("post_media_linker")),
            [
                self.hsf.name
            ]
        )

        self.assertEqual(
            list(otio.hooks.scripts_attached_to("post_adapter_write")),
            [
                self.post_hsf.name
            ]
        )

        tl = otio.schema.Timeline()
        result = otio.hooks.run("pre_adapter_write", tl, TEST_METADATA)
        self.assertEqual(result.name, POST_RUN_NAME)
        self.assertEqual(result.metadata, TEST_METADATA)

        # test deleting all the functions
        del otio.hooks.scripts_attached_to("pre_adapter_write")[:]

        tl = otio.schema.Timeline()
        tl.name = "ORIGINAL"
        result = otio.hooks.run("pre_adapter_write", tl, TEST_METADATA)
        self.assertEqual(result.name, "ORIGINAL")


if __name__ == '__main__':
    unittest.main()
